// include: shell.js
// The Module object: Our interface to the outside world. We import
// and export values on it. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(moduleArg) => Promise<Module>
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to check if Module already exists (e.g. case 3 above).
// Substitution will be replaced with actual code on later stage of the build,
// this way Closure Compiler will not mangle it (e.g. case 4. above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module = typeof libopenmpt != "undefined" ? libopenmpt : {};

// Determine the runtime environment we are in. You can customize this by
// setting the ENVIRONMENT setting at compile time (see settings.js).
// Attempt to auto-detect the environment
var ENVIRONMENT_IS_WEB = typeof window == "object";

var ENVIRONMENT_IS_WORKER = typeof WorkerGlobalScope != "undefined";

// N.b. Electron.js environment is simultaneously a NODE-environment, but
// also a web environment.
var ENVIRONMENT_IS_NODE = typeof process == "object" && process.versions?.node && process.type != "renderer";

// --pre-jses are emitted after the Module integration code, so that they can
// refer to Module (if they choose; they can also define Module)
var arguments_ = [];

var thisProgram = "./this.program";

var quit_ = (status, toThrow) => {
  throw toThrow;
};

// In MODULARIZE mode _scriptName needs to be captured already at the very top of the page immediately when the page is parsed, so it is generated there
// before the page load. In non-MODULARIZE modes generate it here.
var _scriptName = typeof document != "undefined" ? document.currentScript?.src : undefined;

if (typeof __filename != "undefined") {
  // Node
  _scriptName = __filename;
} else if (ENVIRONMENT_IS_WORKER) {
  _scriptName = self.location.href;
}

// `/` should be present at the end if `scriptDirectory` is not empty
var scriptDirectory = "";

function locateFile(path) {
  if (Module["locateFile"]) {
    return Module["locateFile"](path, scriptDirectory);
  }
  return scriptDirectory + path;
}

// Hooks that are implemented differently in different runtime environments.
var readAsync, readBinary;

if (ENVIRONMENT_IS_NODE) {
  // These modules will usually be used on Node.js. Load them eagerly to avoid
  // the complexity of lazy-loading.
  var fs = require("fs");
  scriptDirectory = __dirname + "/";
  // include: node_shell_read.js
  readBinary = filename => {
    // We need to re-wrap `file://` strings to URLs.
    filename = isFileURI(filename) ? new URL(filename) : filename;
    var ret = fs.readFileSync(filename);
    return ret;
  };
  readAsync = async (filename, binary = true) => {
    // See the comment in the `readBinary` function.
    filename = isFileURI(filename) ? new URL(filename) : filename;
    var ret = fs.readFileSync(filename, binary ? undefined : "utf8");
    return ret;
  };
  // end include: node_shell_read.js
  if (process.argv.length > 1) {
    thisProgram = process.argv[1].replace(/\\/g, "/");
  }
  arguments_ = process.argv.slice(2);
  // MODULARIZE will export the module in the proper place outside, we don't need to export here
  if (typeof module != "undefined") {
    module["exports"] = Module;
  }
  quit_ = (status, toThrow) => {
    process.exitCode = status;
    throw toThrow;
  };
} else // Note that this includes Node.js workers when relevant (pthreads is enabled).
// Node.js workers are detected as a combination of ENVIRONMENT_IS_WORKER and
// ENVIRONMENT_IS_NODE.
if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  try {
    scriptDirectory = new URL(".", _scriptName).href;
  } catch {}
  {
    // include: web_or_worker_shell_read.js
    if (ENVIRONMENT_IS_WORKER) {
      readBinary = url => {
        var xhr = new XMLHttpRequest;
        xhr.open("GET", url, false);
        xhr.responseType = "arraybuffer";
        xhr.send(null);
        return new Uint8Array(/** @type{!ArrayBuffer} */ (xhr.response));
      };
    }
    readAsync = async url => {
      // Fetch has some additional restrictions over XHR, like it can't be used on a file:// url.
      // See https://github.com/github/fetch/pull/92#issuecomment-140665932
      // Cordova or Electron apps are typically loaded from a file:// url.
      // So use XHR on webview if URL is a file URL.
      if (isFileURI(url)) {
        return new Promise((resolve, reject) => {
          var xhr = new XMLHttpRequest;
          xhr.open("GET", url, true);
          xhr.responseType = "arraybuffer";
          xhr.onload = () => {
            if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) {
              // file URLs can return 0
              resolve(xhr.response);
              return;
            }
            reject(xhr.status);
          };
          xhr.onerror = reject;
          xhr.send(null);
        });
      }
      var response = await fetch(url, {
        credentials: "same-origin"
      });
      if (response.ok) {
        return response.arrayBuffer();
      }
      throw new Error(response.status + " : " + response.url);
    };
  }
} else {}

var out = console.log.bind(console);

var err = console.error.bind(console);

// end include: shell.js
// include: preamble.js
// === Preamble library stuff ===
// Documentation for the public APIs defined in this file must be updated in:
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at:
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html
var wasmBinary;

// Wasm globals
//========================================
// Runtime essentials
//========================================
// whether we are quitting the application. no code should run after this.
// set in exit() and abort()
var ABORT = false;

// set by exit() and abort().  Passed to 'onExit' handler.
// NOTE: This is also used as the process return code code in shell environments
// but only when noExitRuntime is false.
var EXITSTATUS;

/**
 * Indicates whether filename is delivered via file protocol (as opposed to http/https)
 * @noinline
 */ var isFileURI = filename => filename.startsWith("file://");

// include: runtime_common.js
// include: runtime_stack_check.js
// end include: runtime_stack_check.js
// include: runtime_exceptions.js
// end include: runtime_exceptions.js
// include: runtime_debug.js
// end include: runtime_debug.js
// Memory management
var wasmMemory;

var /** @type {!Int8Array} */ HEAP8, /** @type {!Uint8Array} */ HEAPU8, /** @type {!Int16Array} */ HEAP16, /** @type {!Uint16Array} */ HEAPU16, /** @type {!Int32Array} */ HEAP32, /** @type {!Uint32Array} */ HEAPU32, /** @type {!Float32Array} */ HEAPF32, /** @type {!Float64Array} */ HEAPF64;

// BigInt64Array type is not correctly defined in closure
var /** not-@type {!BigInt64Array} */ HEAP64, /* BigUint64Array type is not correctly defined in closure
/** not-@type {!BigUint64Array} */ HEAPU64;

var runtimeInitialized = false;

function updateMemoryViews() {
  var b = wasmMemory.buffer;
  HEAP8 = new Int8Array(b);
  HEAP16 = new Int16Array(b);
  HEAPU8 = new Uint8Array(b);
  HEAPU16 = new Uint16Array(b);
  HEAP32 = new Int32Array(b);
  HEAPU32 = new Uint32Array(b);
  HEAPF32 = new Float32Array(b);
  HEAPF64 = new Float64Array(b);
  HEAP64 = new BigInt64Array(b);
  HEAPU64 = new BigUint64Array(b);
}

// include: memoryprofiler.js
// end include: memoryprofiler.js
// end include: runtime_common.js
function preRun() {
  if (Module["preRun"]) {
    if (typeof Module["preRun"] == "function") Module["preRun"] = [ Module["preRun"] ];
    while (Module["preRun"].length) {
      addOnPreRun(Module["preRun"].shift());
    }
  }
  // Begin ATPRERUNS hooks
  callRuntimeCallbacks(onPreRuns);
}

function initRuntime() {
  runtimeInitialized = true;
  // Begin ATINITS hooks
  if (!Module["noFSInit"] && !FS.initialized) FS.init();
  TTY.init();
  // End ATINITS hooks
  wasmExports["ya"]();
  // Begin ATPOSTCTORS hooks
  FS.ignorePermissions = false;
}

function postRun() {
  // PThreads reuse the runtime from the main thread.
  if (Module["postRun"]) {
    if (typeof Module["postRun"] == "function") Module["postRun"] = [ Module["postRun"] ];
    while (Module["postRun"].length) {
      addOnPostRun(Module["postRun"].shift());
    }
  }
  // Begin ATPOSTRUNS hooks
  callRuntimeCallbacks(onPostRuns);
}

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// Module.preRun (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;

var dependenciesFulfilled = null;

// overridden to take different actions when all run dependencies are fulfilled
function addRunDependency(id) {
  runDependencies++;
  Module["monitorRunDependencies"]?.(runDependencies);
}

function removeRunDependency(id) {
  runDependencies--;
  Module["monitorRunDependencies"]?.(runDependencies);
  if (runDependencies == 0) {
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback();
    }
  }
}

/** @param {string|number=} what */ function abort(what) {
  Module["onAbort"]?.(what);
  what = "Aborted(" + what + ")";
  // TODO(sbc): Should we remove printing and leave it up to whoever
  // catches the exception?
  err(what);
  ABORT = true;
  what += ". Build with -sASSERTIONS for more info.";
  // Use a wasm runtime error, because a JS error might be seen as a foreign
  // exception, which means we'd run destructors on it. We need the error to
  // simply make the program stop.
  // FIXME This approach does not work in Wasm EH because it currently does not assume
  // all RuntimeErrors are from traps; it decides whether a RuntimeError is from
  // a trap or not based on a hidden field within the object. So at the moment
  // we don't have a way of throwing a wasm trap from JS. TODO Make a JS API that
  // allows this in the wasm spec.
  // Suppress closure compiler warning here. Closure compiler's builtin extern
  // definition for WebAssembly.RuntimeError claims it takes no arguments even
  // though it can.
  // TODO(https://github.com/google/closure-compiler/pull/3913): Remove if/when upstream closure gets fixed.
  /** @suppress {checkTypes} */ var e = new WebAssembly.RuntimeError(what);
  // Throw the error whether or not MODULARIZE is set because abort is used
  // in code paths apart from instantiation where an exception is expected
  // to be thrown when abort is called.
  throw e;
}

var wasmBinaryFile;

function findWasmBinary() {
  return locateFile("libopenmpt.wasm");
}

function getBinarySync(file) {
  if (file == wasmBinaryFile && wasmBinary) {
    return new Uint8Array(wasmBinary);
  }
  if (readBinary) {
    return readBinary(file);
  }
  throw "both async and sync fetching of the wasm failed";
}

async function getWasmBinary(binaryFile) {
  // If we don't have the binary yet, load it asynchronously using readAsync.
  if (!wasmBinary) {
    // Fetch the binary using readAsync
    try {
      var response = await readAsync(binaryFile);
      return new Uint8Array(response);
    } catch {}
  }
  // Otherwise, getBinarySync should be able to get it synchronously
  return getBinarySync(binaryFile);
}

async function instantiateArrayBuffer(binaryFile, imports) {
  try {
    var binary = await getWasmBinary(binaryFile);
    var instance = await WebAssembly.instantiate(binary, imports);
    return instance;
  } catch (reason) {
    err(`failed to asynchronously prepare wasm: ${reason}`);
    abort(reason);
  }
}

async function instantiateAsync(binary, binaryFile, imports) {
  if (!binary && typeof WebAssembly.instantiateStreaming == "function" && !isFileURI(binaryFile) && !ENVIRONMENT_IS_NODE) {
    try {
      var response = fetch(binaryFile, {
        credentials: "same-origin"
      });
      var instantiationResult = await WebAssembly.instantiateStreaming(response, imports);
      return instantiationResult;
    } catch (reason) {
      // We expect the most common failure cause to be a bad MIME type for the binary,
      // in which case falling back to ArrayBuffer instantiation should work.
      err(`wasm streaming compile failed: ${reason}`);
      err("falling back to ArrayBuffer instantiation");
    }
  }
  return instantiateArrayBuffer(binaryFile, imports);
}

function getWasmImports() {
  // prepare imports
  return {
    "a": wasmImports
  };
}

// Create the wasm instance.
// Receives the wasm imports, returns the exports.
async function createWasm() {
  // Load the wasm module and create an instance of using native support in the JS engine.
  // handle a generated wasm instance, receiving its exports and
  // performing other necessary setup
  /** @param {WebAssembly.Module=} module*/ function receiveInstance(instance, module) {
    wasmExports = instance.exports;
    wasmMemory = wasmExports["xa"];
    updateMemoryViews();
    wasmTable = wasmExports["af"];
    assignWasmExports(wasmExports);
    removeRunDependency("wasm-instantiate");
    return wasmExports;
  }
  // wait for the pthread pool (if any)
  addRunDependency("wasm-instantiate");
  // Prefer streaming instantiation if available.
  function receiveInstantiationResult(result) {
    // 'result' is a ResultObject object which has both the module and instance.
    // receiveInstance() will swap in the exports (to Module.asm) so they can be called
    // TODO: Due to Closure regression https://github.com/google/closure-compiler/issues/3193, the above line no longer optimizes out down to the following line.
    // When the regression is fixed, can restore the above PTHREADS-enabled path.
    return receiveInstance(result["instance"]);
  }
  var info = getWasmImports();
  // User shell pages can write their own Module.instantiateWasm = function(imports, successCallback) callback
  // to manually instantiate the Wasm module themselves. This allows pages to
  // run the instantiation parallel to any other async startup actions they are
  // performing.
  // Also pthreads and wasm workers initialize the wasm instance through this
  // path.
  if (Module["instantiateWasm"]) {
    return new Promise((resolve, reject) => {
      Module["instantiateWasm"](info, (mod, inst) => {
        resolve(receiveInstance(mod, inst));
      });
    });
  }
  wasmBinaryFile ??= findWasmBinary();
  var result = await instantiateAsync(wasmBinary, wasmBinaryFile, info);
  var exports = receiveInstantiationResult(result);
  return exports;
}

// end include: preamble.js
// Begin JS library code
class ExitStatus {
  name="ExitStatus";
  constructor(status) {
    this.message = `Program terminated with exit(${status})`;
    this.status = status;
  }
}

var callRuntimeCallbacks = callbacks => {
  while (callbacks.length > 0) {
    // Pass the module as the first argument.
    callbacks.shift()(Module);
  }
};

var onPostRuns = [];

var addOnPostRun = cb => onPostRuns.push(cb);

var onPreRuns = [];

var addOnPreRun = cb => onPreRuns.push(cb);

var noExitRuntime = true;

var stackRestore = val => __emscripten_stack_restore(val);

var stackSave = () => _emscripten_stack_get_current();

var UTF8Decoder = new TextDecoder;

/**
     * Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the
     * emscripten HEAP, returns a copy of that string as a Javascript String object.
     *
     * @param {number} ptr
     * @param {number=} maxBytesToRead - An optional length that specifies the
     *   maximum number of bytes to read. You can omit this parameter to scan the
     *   string until the first 0 byte. If maxBytesToRead is passed, and the string
     *   at [ptr, ptr+maxBytesToReadr[ contains a null byte in the middle, then the
     *   string will cut short at that byte index (i.e. maxBytesToRead will not
     *   produce a string of exact length [ptr, ptr+maxBytesToRead[) N.B. mixing
     *   frequent uses of UTF8ToString() with and without maxBytesToRead may throw
     *   JS JIT optimizations off, so it is worth to consider consistently using one
     * @return {string}
     */ var UTF8ToString = (ptr, maxBytesToRead) => {
  if (!ptr) return "";
  var maxPtr = ptr + maxBytesToRead;
  for (var end = ptr; !(end >= maxPtr) && HEAPU8[end]; ) ++end;
  return UTF8Decoder.decode(HEAPU8.subarray(ptr, end));
};

var ___assert_fail = (condition, filename, line, func) => abort(`Assertion failed: ${UTF8ToString(condition)}, at: ` + [ filename ? UTF8ToString(filename) : "unknown filename", line, func ? UTF8ToString(func) : "unknown function" ]);

var exceptionCaught = [];

var uncaughtExceptionCount = 0;

var ___cxa_begin_catch = ptr => {
  var info = new ExceptionInfo(ptr);
  if (!info.get_caught()) {
    info.set_caught(true);
    uncaughtExceptionCount--;
  }
  info.set_rethrown(false);
  exceptionCaught.push(info);
  ___cxa_increment_exception_refcount(ptr);
  return ___cxa_get_exception_ptr(ptr);
};

var exceptionLast = 0;

var ___cxa_end_catch = () => {
  // Clear state flag.
  _setThrew(0, 0);
  // Call destructor if one is registered then clear it.
  var info = exceptionCaught.pop();
  ___cxa_decrement_exception_refcount(info.excPtr);
  exceptionLast = 0;
};

class ExceptionInfo {
  // excPtr - Thrown object pointer to wrap. Metadata pointer is calculated from it.
  constructor(excPtr) {
    this.excPtr = excPtr;
    this.ptr = excPtr - 24;
  }
  set_type(type) {
    HEAPU32[(((this.ptr) + (4)) >> 2)] = type;
  }
  get_type() {
    return HEAPU32[(((this.ptr) + (4)) >> 2)];
  }
  set_destructor(destructor) {
    HEAPU32[(((this.ptr) + (8)) >> 2)] = destructor;
  }
  get_destructor() {
    return HEAPU32[(((this.ptr) + (8)) >> 2)];
  }
  set_caught(caught) {
    caught = caught ? 1 : 0;
    HEAP8[(this.ptr) + (12)] = caught;
  }
  get_caught() {
    return HEAP8[(this.ptr) + (12)] != 0;
  }
  set_rethrown(rethrown) {
    rethrown = rethrown ? 1 : 0;
    HEAP8[(this.ptr) + (13)] = rethrown;
  }
  get_rethrown() {
    return HEAP8[(this.ptr) + (13)] != 0;
  }
  // Initialize native structure fields. Should be called once after allocated.
  init(type, destructor) {
    this.set_adjusted_ptr(0);
    this.set_type(type);
    this.set_destructor(destructor);
  }
  set_adjusted_ptr(adjustedPtr) {
    HEAPU32[(((this.ptr) + (16)) >> 2)] = adjustedPtr;
  }
  get_adjusted_ptr() {
    return HEAPU32[(((this.ptr) + (16)) >> 2)];
  }
}

var setTempRet0 = val => __emscripten_tempret_set(val);

var findMatchingCatch = args => {
  var thrown = exceptionLast;
  if (!thrown) {
    // just pass through the null ptr
    setTempRet0(0);
    return 0;
  }
  var info = new ExceptionInfo(thrown);
  info.set_adjusted_ptr(thrown);
  var thrownType = info.get_type();
  if (!thrownType) {
    // just pass through the thrown ptr
    setTempRet0(0);
    return thrown;
  }
  // can_catch receives a **, add indirection
  // The different catch blocks are denoted by different types.
  // Due to inheritance, those types may not precisely match the
  // type of the thrown object. Find one which matches, and
  // return the type of the catch block which should be called.
  for (var caughtType of args) {
    if (caughtType === 0 || caughtType === thrownType) {
      // Catch all clause matched or exactly the same type is caught
      break;
    }
    var adjusted_ptr_addr = info.ptr + 16;
    if (___cxa_can_catch(caughtType, thrownType, adjusted_ptr_addr)) {
      setTempRet0(caughtType);
      return thrown;
    }
  }
  setTempRet0(thrownType);
  return thrown;
};

var ___cxa_find_matching_catch_17 = (arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14) => findMatchingCatch([ arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14 ]);

var ___cxa_find_matching_catch_2 = () => findMatchingCatch([]);

var ___cxa_find_matching_catch_3 = arg0 => findMatchingCatch([ arg0 ]);

var ___cxa_find_matching_catch_4 = (arg0, arg1) => findMatchingCatch([ arg0, arg1 ]);

var ___cxa_find_matching_catch_6 = (arg0, arg1, arg2, arg3) => findMatchingCatch([ arg0, arg1, arg2, arg3 ]);

var ___cxa_rethrow = () => {
  var info = exceptionCaught.pop();
  if (!info) {
    abort("no exception to throw");
  }
  var ptr = info.excPtr;
  if (!info.get_rethrown()) {
    // Only pop if the corresponding push was through rethrow_primary_exception
    exceptionCaught.push(info);
    info.set_rethrown(true);
    info.set_caught(false);
    uncaughtExceptionCount++;
  }
  exceptionLast = ptr;
  throw exceptionLast;
};

var ___cxa_throw = (ptr, type, destructor) => {
  var info = new ExceptionInfo(ptr);
  // Initialize ExceptionInfo content after it was allocated in __cxa_allocate_exception.
  info.init(type, destructor);
  exceptionLast = ptr;
  uncaughtExceptionCount++;
  throw exceptionLast;
};

var ___cxa_uncaught_exceptions = () => uncaughtExceptionCount;

var ___resumeException = ptr => {
  if (!exceptionLast) {
    exceptionLast = ptr;
  }
  throw exceptionLast;
};

var __abort_js = () => abort("");

var runtimeKeepaliveCounter = 0;

var __emscripten_runtime_keepalive_clear = () => {
  noExitRuntime = false;
  runtimeKeepaliveCounter = 0;
};

var timers = {};

var handleException = e => {
  // Certain exception types we do not treat as errors since they are used for
  // internal control flow.
  // 1. ExitStatus, which is thrown by exit()
  // 2. "unwind", which is thrown by emscripten_unwind_to_js_event_loop() and others
  //    that wish to return to JS event loop.
  if (e instanceof ExitStatus || e == "unwind") {
    return EXITSTATUS;
  }
  quit_(1, e);
};

var keepRuntimeAlive = () => noExitRuntime || runtimeKeepaliveCounter > 0;

var _proc_exit = code => {
  EXITSTATUS = code;
  if (!keepRuntimeAlive()) {
    Module["onExit"]?.(code);
    ABORT = true;
  }
  quit_(code, new ExitStatus(code));
};

/** @suppress {duplicate } */ /** @param {boolean|number=} implicit */ var exitJS = (status, implicit) => {
  EXITSTATUS = status;
  _proc_exit(status);
};

var _exit = exitJS;

var maybeExit = () => {
  if (!keepRuntimeAlive()) {
    try {
      _exit(EXITSTATUS);
    } catch (e) {
      handleException(e);
    }
  }
};

var callUserCallback = func => {
  if (ABORT) {
    return;
  }
  try {
    func();
    maybeExit();
  } catch (e) {
    handleException(e);
  }
};

var _emscripten_get_now = () => performance.now();

var __setitimer_js = (which, timeout_ms) => {
  // First, clear any existing timer.
  if (timers[which]) {
    clearTimeout(timers[which].id);
    delete timers[which];
  }
  // A timeout of zero simply cancels the current timeout so we have nothing
  // more to do.
  if (!timeout_ms) return 0;
  var id = setTimeout(() => {
    delete timers[which];
    callUserCallback(() => __emscripten_timeout(which, _emscripten_get_now()));
  }, timeout_ms);
  timers[which] = {
    id,
    timeout_ms
  };
  return 0;
};

var stringToUTF8Array = (str, heap, outIdx, maxBytesToWrite) => {
  // Parameter maxBytesToWrite is not optional. Negative values, 0, null,
  // undefined and false each don't write out any bytes.
  if (!(maxBytesToWrite > 0)) return 0;
  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1;
  // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description
    // and https://www.ietf.org/rfc/rfc2279.txt
    // and https://tools.ietf.org/html/rfc3629
    var u = str.codePointAt(i);
    if (u <= 127) {
      if (outIdx >= endIdx) break;
      heap[outIdx++] = u;
    } else if (u <= 2047) {
      if (outIdx + 1 >= endIdx) break;
      heap[outIdx++] = 192 | (u >> 6);
      heap[outIdx++] = 128 | (u & 63);
    } else if (u <= 65535) {
      if (outIdx + 2 >= endIdx) break;
      heap[outIdx++] = 224 | (u >> 12);
      heap[outIdx++] = 128 | ((u >> 6) & 63);
      heap[outIdx++] = 128 | (u & 63);
    } else {
      if (outIdx + 3 >= endIdx) break;
      heap[outIdx++] = 240 | (u >> 18);
      heap[outIdx++] = 128 | ((u >> 12) & 63);
      heap[outIdx++] = 128 | ((u >> 6) & 63);
      heap[outIdx++] = 128 | (u & 63);
      // Gotcha: if codePoint is over 0xFFFF, it is represented as a surrogate pair in UTF-16.
      // We need to manually skip over the second code unit for correct iteration.
      i++;
    }
  }
  // Null-terminate the pointer to the buffer.
  heap[outIdx] = 0;
  return outIdx - startIdx;
};

var stringToUTF8 = (str, outPtr, maxBytesToWrite) => stringToUTF8Array(str, HEAPU8, outPtr, maxBytesToWrite);

var __tzset_js = (timezone, daylight, std_name, dst_name) => {
  // TODO: Use (malleable) environment variables instead of system settings.
  var currentYear = (new Date).getFullYear();
  var winter = new Date(currentYear, 0, 1);
  var summer = new Date(currentYear, 6, 1);
  var winterOffset = winter.getTimezoneOffset();
  var summerOffset = summer.getTimezoneOffset();
  // Local standard timezone offset. Local standard time is not adjusted for
  // daylight savings.  This code uses the fact that getTimezoneOffset returns
  // a greater value during Standard Time versus Daylight Saving Time (DST).
  // Thus it determines the expected output during Standard Time, and it
  // compares whether the output of the given date the same (Standard) or less
  // (DST).
  var stdTimezoneOffset = Math.max(winterOffset, summerOffset);
  // timezone is specified as seconds west of UTC ("The external variable
  // `timezone` shall be set to the difference, in seconds, between
  // Coordinated Universal Time (UTC) and local standard time."), the same
  // as returned by stdTimezoneOffset.
  // See http://pubs.opengroup.org/onlinepubs/009695399/functions/tzset.html
  HEAPU32[((timezone) >> 2)] = stdTimezoneOffset * 60;
  HEAP32[((daylight) >> 2)] = Number(winterOffset != summerOffset);
  var extractZone = timezoneOffset => {
    // Why inverse sign?
    // Read here https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/getTimezoneOffset
    var sign = timezoneOffset >= 0 ? "-" : "+";
    var absOffset = Math.abs(timezoneOffset);
    var hours = String(Math.floor(absOffset / 60)).padStart(2, "0");
    var minutes = String(absOffset % 60).padStart(2, "0");
    return `UTC${sign}${hours}${minutes}`;
  };
  var winterName = extractZone(winterOffset);
  var summerName = extractZone(summerOffset);
  if (summerOffset < winterOffset) {
    // Northern hemisphere
    stringToUTF8(winterName, std_name, 17);
    stringToUTF8(summerName, dst_name, 17);
  } else {
    stringToUTF8(winterName, dst_name, 17);
    stringToUTF8(summerName, std_name, 17);
  }
};

var _emscripten_date_now = () => Date.now();

var nowIsMonotonic = 1;

var checkWasiClock = clock_id => clock_id >= 0 && clock_id <= 3;

var INT53_MAX = 9007199254740992;

var INT53_MIN = -9007199254740992;

var bigintToI53Checked = num => (num < INT53_MIN || num > INT53_MAX) ? NaN : Number(num);

function _clock_time_get(clk_id, ignored_precision, ptime) {
  ignored_precision = bigintToI53Checked(ignored_precision);
  if (!checkWasiClock(clk_id)) {
    return 28;
  }
  var now;
  // all wasi clocks but realtime are monotonic
  if (clk_id === 0) {
    now = _emscripten_date_now();
  } else if (nowIsMonotonic) {
    now = _emscripten_get_now();
  } else {
    return 52;
  }
  // "now" is in ms, and wasi times are in ns.
  var nsec = Math.round(now * 1e3 * 1e3);
  HEAP64[((ptime) >> 3)] = BigInt(nsec);
  return 0;
}

var getHeapMax = () => // Stay one Wasm page short of 4GB: while e.g. Chrome is able to allocate
// full 4GB Wasm memories, the size will wrap back to 0 bytes in Wasm side
// for any code that deals with heap sizes, which would require special
// casing all heap size related code to treat 0 specially.
2147483648;

var alignMemory = (size, alignment) => Math.ceil(size / alignment) * alignment;

var growMemory = size => {
  var b = wasmMemory.buffer;
  var pages = ((size - b.byteLength + 65535) / 65536) | 0;
  try {
    // round size grow request up to wasm page size (fixed 64KB per spec)
    wasmMemory.grow(pages);
    // .grow() takes a delta compared to the previous size
    updateMemoryViews();
    return 1;
  } catch (e) {}
};

var _emscripten_resize_heap = requestedSize => {
  var oldSize = HEAPU8.length;
  // With CAN_ADDRESS_2GB or MEMORY64, pointers are already unsigned.
  requestedSize >>>= 0;
  // With multithreaded builds, races can happen (another thread might increase the size
  // in between), so return a failure, and let the caller retry.
  // Memory resize rules:
  // 1.  Always increase heap size to at least the requested size, rounded up
  //     to next page multiple.
  // 2a. If MEMORY_GROWTH_LINEAR_STEP == -1, excessively resize the heap
  //     geometrically: increase the heap size according to
  //     MEMORY_GROWTH_GEOMETRIC_STEP factor (default +20%), At most
  //     overreserve by MEMORY_GROWTH_GEOMETRIC_CAP bytes (default 96MB).
  // 2b. If MEMORY_GROWTH_LINEAR_STEP != -1, excessively resize the heap
  //     linearly: increase the heap size by at least
  //     MEMORY_GROWTH_LINEAR_STEP bytes.
  // 3.  Max size for the heap is capped at 2048MB-WASM_PAGE_SIZE, or by
  //     MAXIMUM_MEMORY, or by ASAN limit, depending on which is smallest
  // 4.  If we were unable to allocate as much memory, it may be due to
  //     over-eager decision to excessively reserve due to (3) above.
  //     Hence if an allocation fails, cut down on the amount of excess
  //     growth, in an attempt to succeed to perform a smaller allocation.
  // A limit is set for how much we can grow. We should not exceed that
  // (the wasm binary specifies it, so if we tried, we'd fail anyhow).
  var maxHeapSize = getHeapMax();
  if (requestedSize > maxHeapSize) {
    return false;
  }
  // Loop through potential heap size increases. If we attempt a too eager
  // reservation that fails, cut down on the attempted size and reserve a
  // smaller bump instead. (max 3 times, chosen somewhat arbitrarily)
  for (var cutDown = 1; cutDown <= 4; cutDown *= 2) {
    var overGrownHeapSize = oldSize * (1 + .2 / cutDown);
    // ensure geometric growth
    // but limit overreserving (default to capping at +96MB overgrowth at most)
    overGrownHeapSize = Math.min(overGrownHeapSize, requestedSize + 100663296);
    var newSize = Math.min(maxHeapSize, alignMemory(Math.max(requestedSize, overGrownHeapSize), 65536));
    var replacement = growMemory(newSize);
    if (replacement) {
      return true;
    }
  }
  return false;
};

var ENV = {};

var getExecutableName = () => thisProgram || "./this.program";

var getEnvStrings = () => {
  if (!getEnvStrings.strings) {
    // Default values.
    // Browser language detection #8751
    var lang = ((typeof navigator == "object" && navigator.language) || "C").replace("-", "_") + ".UTF-8";
    var env = {
      "USER": "web_user",
      "LOGNAME": "web_user",
      "PATH": "/",
      "PWD": "/",
      "HOME": "/home/web_user",
      "LANG": lang,
      "_": getExecutableName()
    };
    // Apply the user-provided values, if any.
    for (var x in ENV) {
      // x is a key in ENV; if ENV[x] is undefined, that means it was
      // explicitly set to be so. We allow user code to do that to
      // force variables with default values to remain unset.
      if (ENV[x] === undefined) delete env[x]; else env[x] = ENV[x];
    }
    var strings = [];
    for (var x in env) {
      strings.push(`${x}=${env[x]}`);
    }
    getEnvStrings.strings = strings;
  }
  return getEnvStrings.strings;
};

var _environ_get = (__environ, environ_buf) => {
  var bufSize = 0;
  var envp = 0;
  for (var string of getEnvStrings()) {
    var ptr = environ_buf + bufSize;
    HEAPU32[(((__environ) + (envp)) >> 2)] = ptr;
    bufSize += stringToUTF8(string, ptr, Infinity) + 1;
    envp += 4;
  }
  return 0;
};

var lengthBytesUTF8 = str => {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code
    // unit, not a Unicode code point of the character! So decode
    // UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var c = str.charCodeAt(i);
    // possibly a lead surrogate
    if (c <= 127) {
      len++;
    } else if (c <= 2047) {
      len += 2;
    } else if (c >= 55296 && c <= 57343) {
      len += 4;
      ++i;
    } else {
      len += 3;
    }
  }
  return len;
};

var _environ_sizes_get = (penviron_count, penviron_buf_size) => {
  var strings = getEnvStrings();
  HEAPU32[((penviron_count) >> 2)] = strings.length;
  var bufSize = 0;
  for (var string of strings) {
    bufSize += lengthBytesUTF8(string) + 1;
  }
  HEAPU32[((penviron_buf_size) >> 2)] = bufSize;
  return 0;
};

var PATH = {
  isAbs: path => path.charAt(0) === "/",
  splitPath: filename => {
    var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
    return splitPathRe.exec(filename).slice(1);
  },
  normalizeArray: (parts, allowAboveRoot) => {
    // if the path tries to go above the root, `up` ends up > 0
    var up = 0;
    for (var i = parts.length - 1; i >= 0; i--) {
      var last = parts[i];
      if (last === ".") {
        parts.splice(i, 1);
      } else if (last === "..") {
        parts.splice(i, 1);
        up++;
      } else if (up) {
        parts.splice(i, 1);
        up--;
      }
    }
    // if the path is allowed to go above the root, restore leading ..s
    if (allowAboveRoot) {
      for (;up; up--) {
        parts.unshift("..");
      }
    }
    return parts;
  },
  normalize: path => {
    var isAbsolute = PATH.isAbs(path), trailingSlash = path.slice(-1) === "/";
    // Normalize the path
    path = PATH.normalizeArray(path.split("/").filter(p => !!p), !isAbsolute).join("/");
    if (!path && !isAbsolute) {
      path = ".";
    }
    if (path && trailingSlash) {
      path += "/";
    }
    return (isAbsolute ? "/" : "") + path;
  },
  dirname: path => {
    var result = PATH.splitPath(path), root = result[0], dir = result[1];
    if (!root && !dir) {
      // No dirname whatsoever
      return ".";
    }
    if (dir) {
      // It has a dirname, strip trailing slash
      dir = dir.slice(0, -1);
    }
    return root + dir;
  },
  basename: path => path && path.match(/([^\/]+|\/)\/*$/)[1],
  join: (...paths) => PATH.normalize(paths.join("/")),
  join2: (l, r) => PATH.normalize(l + "/" + r)
};

var initRandomFill = () => {
  // This block is not needed on v19+ since crypto.getRandomValues is builtin
  if (ENVIRONMENT_IS_NODE) {
    var nodeCrypto = require("crypto");
    return view => nodeCrypto.randomFillSync(view);
  }
  return view => crypto.getRandomValues(view);
};

var randomFill = view => {
  // Lazily init on the first invocation.
  (randomFill = initRandomFill())(view);
};

var PATH_FS = {
  resolve: (...args) => {
    var resolvedPath = "", resolvedAbsolute = false;
    for (var i = args.length - 1; i >= -1 && !resolvedAbsolute; i--) {
      var path = (i >= 0) ? args[i] : FS.cwd();
      // Skip empty and invalid entries
      if (typeof path != "string") {
        throw new TypeError("Arguments to path.resolve must be strings");
      } else if (!path) {
        return "";
      }
      resolvedPath = path + "/" + resolvedPath;
      resolvedAbsolute = PATH.isAbs(path);
    }
    // At this point the path should be resolved to a full absolute path, but
    // handle relative paths to be safe (might happen when process.cwd() fails)
    resolvedPath = PATH.normalizeArray(resolvedPath.split("/").filter(p => !!p), !resolvedAbsolute).join("/");
    return ((resolvedAbsolute ? "/" : "") + resolvedPath) || ".";
  },
  relative: (from, to) => {
    from = PATH_FS.resolve(from).slice(1);
    to = PATH_FS.resolve(to).slice(1);
    function trim(arr) {
      var start = 0;
      for (;start < arr.length; start++) {
        if (arr[start] !== "") break;
      }
      var end = arr.length - 1;
      for (;end >= 0; end--) {
        if (arr[end] !== "") break;
      }
      if (start > end) return [];
      return arr.slice(start, end - start + 1);
    }
    var fromParts = trim(from.split("/"));
    var toParts = trim(to.split("/"));
    var length = Math.min(fromParts.length, toParts.length);
    var samePartsLength = length;
    for (var i = 0; i < length; i++) {
      if (fromParts[i] !== toParts[i]) {
        samePartsLength = i;
        break;
      }
    }
    var outputParts = [];
    for (var i = samePartsLength; i < fromParts.length; i++) {
      outputParts.push("..");
    }
    outputParts = outputParts.concat(toParts.slice(samePartsLength));
    return outputParts.join("/");
  }
};

/**
     * Given a pointer 'idx' to a null-terminated UTF8-encoded string in the given
     * array that contains uint8 values, returns a copy of that string as a
     * Javascript String object.
     * heapOrArray is either a regular array, or a JavaScript typed array view.
     * @param {number=} idx
     * @param {number=} maxBytesToRead
     * @return {string}
     */ var UTF8ArrayToString = (heapOrArray, idx = 0, maxBytesToRead = NaN) => {
  var endIdx = idx + maxBytesToRead;
  var endPtr = idx;
  // TextDecoder needs to know the byte length in advance, it doesn't stop on
  // null terminator by itself.  Also, use the length info to avoid running tiny
  // strings through TextDecoder, since .subarray() allocates garbage.
  // (As a tiny code save trick, compare endPtr against endIdx using a negation,
  // so that undefined/NaN means Infinity)
  while (heapOrArray[endPtr] && !(endPtr >= endIdx)) ++endPtr;
  return UTF8Decoder.decode(heapOrArray.buffer ? heapOrArray.subarray(idx, endPtr) : new Uint8Array(heapOrArray.slice(idx, endPtr)));
};

var FS_stdin_getChar_buffer = [];

/** @type {function(string, boolean=, number=)} */ var intArrayFromString = (stringy, dontAddNull, length) => {
  var len = length > 0 ? length : lengthBytesUTF8(stringy) + 1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
};

var FS_stdin_getChar = () => {
  if (!FS_stdin_getChar_buffer.length) {
    var result = null;
    if (ENVIRONMENT_IS_NODE) {
      // we will read data by chunks of BUFSIZE
      var BUFSIZE = 256;
      var buf = Buffer.alloc(BUFSIZE);
      var bytesRead = 0;
      // For some reason we must suppress a closure warning here, even though
      // fd definitely exists on process.stdin, and is even the proper way to
      // get the fd of stdin,
      // https://github.com/nodejs/help/issues/2136#issuecomment-523649904
      // This started to happen after moving this logic out of library_tty.js,
      // so it is related to the surrounding code in some unclear manner.
      /** @suppress {missingProperties} */ var fd = process.stdin.fd;
      try {
        bytesRead = fs.readSync(fd, buf, 0, BUFSIZE);
      } catch (e) {
        // Cross-platform differences: on Windows, reading EOF throws an
        // exception, but on other OSes, reading EOF returns 0. Uniformize
        // behavior by treating the EOF exception to return 0.
        if (e.toString().includes("EOF")) bytesRead = 0; else throw e;
      }
      if (bytesRead > 0) {
        result = buf.slice(0, bytesRead).toString("utf-8");
      }
    } else if (typeof window != "undefined" && typeof window.prompt == "function") {
      // Browser.
      result = window.prompt("Input: ");
      // returns null on cancel
      if (result !== null) {
        result += "\n";
      }
    } else {}
    if (!result) {
      return null;
    }
    FS_stdin_getChar_buffer = intArrayFromString(result, true);
  }
  return FS_stdin_getChar_buffer.shift();
};

var TTY = {
  ttys: [],
  init() {},
  shutdown() {},
  register(dev, ops) {
    TTY.ttys[dev] = {
      input: [],
      output: [],
      ops
    };
    FS.registerDevice(dev, TTY.stream_ops);
  },
  stream_ops: {
    open(stream) {
      var tty = TTY.ttys[stream.node.rdev];
      if (!tty) {
        throw new FS.ErrnoError(43);
      }
      stream.tty = tty;
      stream.seekable = false;
    },
    close(stream) {
      // flush any pending line data
      stream.tty.ops.fsync(stream.tty);
    },
    fsync(stream) {
      stream.tty.ops.fsync(stream.tty);
    },
    read(stream, buffer, offset, length, pos) {
      if (!stream.tty || !stream.tty.ops.get_char) {
        throw new FS.ErrnoError(60);
      }
      var bytesRead = 0;
      for (var i = 0; i < length; i++) {
        var result;
        try {
          result = stream.tty.ops.get_char(stream.tty);
        } catch (e) {
          throw new FS.ErrnoError(29);
        }
        if (result === undefined && bytesRead === 0) {
          throw new FS.ErrnoError(6);
        }
        if (result === null || result === undefined) break;
        bytesRead++;
        buffer[offset + i] = result;
      }
      if (bytesRead) {
        stream.node.atime = Date.now();
      }
      return bytesRead;
    },
    write(stream, buffer, offset, length, pos) {
      if (!stream.tty || !stream.tty.ops.put_char) {
        throw new FS.ErrnoError(60);
      }
      try {
        for (var i = 0; i < length; i++) {
          stream.tty.ops.put_char(stream.tty, buffer[offset + i]);
        }
      } catch (e) {
        throw new FS.ErrnoError(29);
      }
      if (length) {
        stream.node.mtime = stream.node.ctime = Date.now();
      }
      return i;
    }
  },
  default_tty_ops: {
    get_char(tty) {
      return FS_stdin_getChar();
    },
    put_char(tty, val) {
      if (val === null || val === 10) {
        out(UTF8ArrayToString(tty.output));
        tty.output = [];
      } else {
        if (val != 0) tty.output.push(val);
      }
    },
    fsync(tty) {
      if (tty.output?.length > 0) {
        out(UTF8ArrayToString(tty.output));
        tty.output = [];
      }
    },
    ioctl_tcgets(tty) {
      // typical setting
      return {
        c_iflag: 25856,
        c_oflag: 5,
        c_cflag: 191,
        c_lflag: 35387,
        c_cc: [ 3, 28, 127, 21, 4, 0, 1, 0, 17, 19, 26, 0, 18, 15, 23, 22, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ]
      };
    },
    ioctl_tcsets(tty, optional_actions, data) {
      // currently just ignore
      return 0;
    },
    ioctl_tiocgwinsz(tty) {
      return [ 24, 80 ];
    }
  },
  default_tty1_ops: {
    put_char(tty, val) {
      if (val === null || val === 10) {
        err(UTF8ArrayToString(tty.output));
        tty.output = [];
      } else {
        if (val != 0) tty.output.push(val);
      }
    },
    fsync(tty) {
      if (tty.output?.length > 0) {
        err(UTF8ArrayToString(tty.output));
        tty.output = [];
      }
    }
  }
};

var mmapAlloc = size => {
  abort();
};

var MEMFS = {
  ops_table: null,
  mount(mount) {
    return MEMFS.createNode(null, "/", 16895, 0);
  },
  createNode(parent, name, mode, dev) {
    if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
      // no supported
      throw new FS.ErrnoError(63);
    }
    MEMFS.ops_table ||= {
      dir: {
        node: {
          getattr: MEMFS.node_ops.getattr,
          setattr: MEMFS.node_ops.setattr,
          lookup: MEMFS.node_ops.lookup,
          mknod: MEMFS.node_ops.mknod,
          rename: MEMFS.node_ops.rename,
          unlink: MEMFS.node_ops.unlink,
          rmdir: MEMFS.node_ops.rmdir,
          readdir: MEMFS.node_ops.readdir,
          symlink: MEMFS.node_ops.symlink
        },
        stream: {
          llseek: MEMFS.stream_ops.llseek
        }
      },
      file: {
        node: {
          getattr: MEMFS.node_ops.getattr,
          setattr: MEMFS.node_ops.setattr
        },
        stream: {
          llseek: MEMFS.stream_ops.llseek,
          read: MEMFS.stream_ops.read,
          write: MEMFS.stream_ops.write,
          mmap: MEMFS.stream_ops.mmap,
          msync: MEMFS.stream_ops.msync
        }
      },
      link: {
        node: {
          getattr: MEMFS.node_ops.getattr,
          setattr: MEMFS.node_ops.setattr,
          readlink: MEMFS.node_ops.readlink
        },
        stream: {}
      },
      chrdev: {
        node: {
          getattr: MEMFS.node_ops.getattr,
          setattr: MEMFS.node_ops.setattr
        },
        stream: FS.chrdev_stream_ops
      }
    };
    var node = FS.createNode(parent, name, mode, dev);
    if (FS.isDir(node.mode)) {
      node.node_ops = MEMFS.ops_table.dir.node;
      node.stream_ops = MEMFS.ops_table.dir.stream;
      node.contents = {};
    } else if (FS.isFile(node.mode)) {
      node.node_ops = MEMFS.ops_table.file.node;
      node.stream_ops = MEMFS.ops_table.file.stream;
      node.usedBytes = 0;
      // The actual number of bytes used in the typed array, as opposed to contents.length which gives the whole capacity.
      // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
      // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
      // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
      node.contents = null;
    } else if (FS.isLink(node.mode)) {
      node.node_ops = MEMFS.ops_table.link.node;
      node.stream_ops = MEMFS.ops_table.link.stream;
    } else if (FS.isChrdev(node.mode)) {
      node.node_ops = MEMFS.ops_table.chrdev.node;
      node.stream_ops = MEMFS.ops_table.chrdev.stream;
    }
    node.atime = node.mtime = node.ctime = Date.now();
    // add the new node to the parent
    if (parent) {
      parent.contents[name] = node;
      parent.atime = parent.mtime = parent.ctime = node.atime;
    }
    return node;
  },
  getFileDataAsTypedArray(node) {
    if (!node.contents) return new Uint8Array(0);
    if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes);
    // Make sure to not return excess unused bytes.
    return new Uint8Array(node.contents);
  },
  expandFileStorage(node, newCapacity) {
    var prevCapacity = node.contents ? node.contents.length : 0;
    if (prevCapacity >= newCapacity) return;
    // No need to expand, the storage was already large enough.
    // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
    // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
    // avoid overshooting the allocation cap by a very large margin.
    var CAPACITY_DOUBLING_MAX = 1024 * 1024;
    newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2 : 1.125)) >>> 0);
    if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256);
    // At minimum allocate 256b for each file when expanding.
    var oldContents = node.contents;
    node.contents = new Uint8Array(newCapacity);
    // Allocate new storage.
    if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0);
  },
  resizeFileStorage(node, newSize) {
    if (node.usedBytes == newSize) return;
    if (newSize == 0) {
      node.contents = null;
      // Fully decommit when requesting a resize to zero.
      node.usedBytes = 0;
    } else {
      var oldContents = node.contents;
      node.contents = new Uint8Array(newSize);
      // Allocate new storage.
      if (oldContents) {
        node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes)));
      }
      node.usedBytes = newSize;
    }
  },
  node_ops: {
    getattr(node) {
      var attr = {};
      // device numbers reuse inode numbers.
      attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
      attr.ino = node.id;
      attr.mode = node.mode;
      attr.nlink = 1;
      attr.uid = 0;
      attr.gid = 0;
      attr.rdev = node.rdev;
      if (FS.isDir(node.mode)) {
        attr.size = 4096;
      } else if (FS.isFile(node.mode)) {
        attr.size = node.usedBytes;
      } else if (FS.isLink(node.mode)) {
        attr.size = node.link.length;
      } else {
        attr.size = 0;
      }
      attr.atime = new Date(node.atime);
      attr.mtime = new Date(node.mtime);
      attr.ctime = new Date(node.ctime);
      // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
      //       but this is not required by the standard.
      attr.blksize = 4096;
      attr.blocks = Math.ceil(attr.size / attr.blksize);
      return attr;
    },
    setattr(node, attr) {
      for (const key of [ "mode", "atime", "mtime", "ctime" ]) {
        if (attr[key] != null) {
          node[key] = attr[key];
        }
      }
      if (attr.size !== undefined) {
        MEMFS.resizeFileStorage(node, attr.size);
      }
    },
    lookup(parent, name) {
      throw MEMFS.doesNotExistError;
    },
    mknod(parent, name, mode, dev) {
      return MEMFS.createNode(parent, name, mode, dev);
    },
    rename(old_node, new_dir, new_name) {
      var new_node;
      try {
        new_node = FS.lookupNode(new_dir, new_name);
      } catch (e) {}
      if (new_node) {
        if (FS.isDir(old_node.mode)) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          for (var i in new_node.contents) {
            throw new FS.ErrnoError(55);
          }
        }
        FS.hashRemoveNode(new_node);
      }
      // do the internal rewiring
      delete old_node.parent.contents[old_node.name];
      new_dir.contents[new_name] = old_node;
      old_node.name = new_name;
      new_dir.ctime = new_dir.mtime = old_node.parent.ctime = old_node.parent.mtime = Date.now();
    },
    unlink(parent, name) {
      delete parent.contents[name];
      parent.ctime = parent.mtime = Date.now();
    },
    rmdir(parent, name) {
      var node = FS.lookupNode(parent, name);
      for (var i in node.contents) {
        throw new FS.ErrnoError(55);
      }
      delete parent.contents[name];
      parent.ctime = parent.mtime = Date.now();
    },
    readdir(node) {
      return [ ".", "..", ...Object.keys(node.contents) ];
    },
    symlink(parent, newname, oldpath) {
      var node = MEMFS.createNode(parent, newname, 511 | 40960, 0);
      node.link = oldpath;
      return node;
    },
    readlink(node) {
      if (!FS.isLink(node.mode)) {
        throw new FS.ErrnoError(28);
      }
      return node.link;
    }
  },
  stream_ops: {
    read(stream, buffer, offset, length, position) {
      var contents = stream.node.contents;
      if (position >= stream.node.usedBytes) return 0;
      var size = Math.min(stream.node.usedBytes - position, length);
      if (size > 8 && contents.subarray) {
        // non-trivial, and typed array
        buffer.set(contents.subarray(position, position + size), offset);
      } else {
        for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
      }
      return size;
    },
    write(stream, buffer, offset, length, position, canOwn) {
      // If the buffer is located in main memory (HEAP), and if
      // memory can grow, we can't hold on to references of the
      // memory buffer, as they may get invalidated. That means we
      // need to do copy its contents.
      if (buffer.buffer === HEAP8.buffer) {
        canOwn = false;
      }
      if (!length) return 0;
      var node = stream.node;
      node.mtime = node.ctime = Date.now();
      if (buffer.subarray && (!node.contents || node.contents.subarray)) {
        // This write is from a typed array to a typed array?
        if (canOwn) {
          node.contents = buffer.subarray(offset, offset + length);
          node.usedBytes = length;
          return length;
        } else if (node.usedBytes === 0 && position === 0) {
          // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
          node.contents = buffer.slice(offset, offset + length);
          node.usedBytes = length;
          return length;
        } else if (position + length <= node.usedBytes) {
          // Writing to an already allocated and used subrange of the file?
          node.contents.set(buffer.subarray(offset, offset + length), position);
          return length;
        }
      }
      // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
      MEMFS.expandFileStorage(node, position + length);
      if (node.contents.subarray && buffer.subarray) {
        // Use typed array write which is available.
        node.contents.set(buffer.subarray(offset, offset + length), position);
      } else {
        for (var i = 0; i < length; i++) {
          node.contents[position + i] = buffer[offset + i];
        }
      }
      node.usedBytes = Math.max(node.usedBytes, position + length);
      return length;
    },
    llseek(stream, offset, whence) {
      var position = offset;
      if (whence === 1) {
        position += stream.position;
      } else if (whence === 2) {
        if (FS.isFile(stream.node.mode)) {
          position += stream.node.usedBytes;
        }
      }
      if (position < 0) {
        throw new FS.ErrnoError(28);
      }
      return position;
    },
    mmap(stream, length, position, prot, flags) {
      if (!FS.isFile(stream.node.mode)) {
        throw new FS.ErrnoError(43);
      }
      var ptr;
      var allocated;
      var contents = stream.node.contents;
      // Only make a new copy when MAP_PRIVATE is specified.
      if (!(flags & 2) && contents && contents.buffer === HEAP8.buffer) {
        // We can't emulate MAP_SHARED when the file is not backed by the
        // buffer we're mapping to (e.g. the HEAP buffer).
        allocated = false;
        ptr = contents.byteOffset;
      } else {
        allocated = true;
        ptr = mmapAlloc(length);
        if (!ptr) {
          throw new FS.ErrnoError(48);
        }
        if (contents) {
          // Try to avoid unnecessary slices.
          if (position > 0 || position + length < contents.length) {
            if (contents.subarray) {
              contents = contents.subarray(position, position + length);
            } else {
              contents = Array.prototype.slice.call(contents, position, position + length);
            }
          }
          HEAP8.set(contents, ptr);
        }
      }
      return {
        ptr,
        allocated
      };
    },
    msync(stream, buffer, offset, length, mmapFlags) {
      MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
      // should we check if bytesWritten and length are the same?
      return 0;
    }
  }
};

var asyncLoad = async url => {
  var arrayBuffer = await readAsync(url);
  return new Uint8Array(arrayBuffer);
};

var FS_createDataFile = (...args) => FS.createDataFile(...args);

var getUniqueRunDependency = id => id;

var preloadPlugins = [];

var FS_handledByPreloadPlugin = (byteArray, fullname, finish, onerror) => {
  // Ensure plugins are ready.
  if (typeof Browser != "undefined") Browser.init();
  var handled = false;
  preloadPlugins.forEach(plugin => {
    if (handled) return;
    if (plugin["canHandle"](fullname)) {
      plugin["handle"](byteArray, fullname, finish, onerror);
      handled = true;
    }
  });
  return handled;
};

var FS_createPreloadedFile = (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) => {
  // TODO we should allow people to just pass in a complete filename instead
  // of parent and name being that we just join them anyways
  var fullname = name ? PATH_FS.resolve(PATH.join2(parent, name)) : parent;
  var dep = getUniqueRunDependency(`cp ${fullname}`);
  // might have several active requests for the same fullname
  function processData(byteArray) {
    function finish(byteArray) {
      preFinish?.();
      if (!dontCreateFile) {
        FS_createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
      }
      onload?.();
      removeRunDependency(dep);
    }
    if (FS_handledByPreloadPlugin(byteArray, fullname, finish, () => {
      onerror?.();
      removeRunDependency(dep);
    })) {
      return;
    }
    finish(byteArray);
  }
  addRunDependency(dep);
  if (typeof url == "string") {
    asyncLoad(url).then(processData, onerror);
  } else {
    processData(url);
  }
};

var FS_modeStringToFlags = str => {
  var flagModes = {
    "r": 0,
    "r+": 2,
    "w": 512 | 64 | 1,
    "w+": 512 | 64 | 2,
    "a": 1024 | 64 | 1,
    "a+": 1024 | 64 | 2
  };
  var flags = flagModes[str];
  if (typeof flags == "undefined") {
    throw new Error(`Unknown file open mode: ${str}`);
  }
  return flags;
};

var FS_getMode = (canRead, canWrite) => {
  var mode = 0;
  if (canRead) mode |= 292 | 73;
  if (canWrite) mode |= 146;
  return mode;
};

var FS = {
  root: null,
  mounts: [],
  devices: {},
  streams: [],
  nextInode: 1,
  nameTable: null,
  currentPath: "/",
  initialized: false,
  ignorePermissions: true,
  filesystems: null,
  syncFSRequests: 0,
  readFiles: {},
  ErrnoError: class {
    name="ErrnoError";
    // We set the `name` property to be able to identify `FS.ErrnoError`
    // - the `name` is a standard ECMA-262 property of error objects. Kind of good to have it anyway.
    // - when using PROXYFS, an error can come from an underlying FS
    // as different FS objects have their own FS.ErrnoError each,
    // the test `err instanceof FS.ErrnoError` won't detect an error coming from another filesystem, causing bugs.
    // we'll use the reliable test `err.name == "ErrnoError"` instead
    constructor(errno) {
      this.errno = errno;
    }
  },
  FSStream: class {
    shared={};
    get object() {
      return this.node;
    }
    set object(val) {
      this.node = val;
    }
    get isRead() {
      return (this.flags & 2097155) !== 1;
    }
    get isWrite() {
      return (this.flags & 2097155) !== 0;
    }
    get isAppend() {
      return (this.flags & 1024);
    }
    get flags() {
      return this.shared.flags;
    }
    set flags(val) {
      this.shared.flags = val;
    }
    get position() {
      return this.shared.position;
    }
    set position(val) {
      this.shared.position = val;
    }
  },
  FSNode: class {
    node_ops={};
    stream_ops={};
    readMode=292 | 73;
    writeMode=146;
    mounted=null;
    constructor(parent, name, mode, rdev) {
      if (!parent) {
        parent = this;
      }
      this.parent = parent;
      this.mount = parent.mount;
      this.id = FS.nextInode++;
      this.name = name;
      this.mode = mode;
      this.rdev = rdev;
      this.atime = this.mtime = this.ctime = Date.now();
    }
    get read() {
      return (this.mode & this.readMode) === this.readMode;
    }
    set read(val) {
      val ? this.mode |= this.readMode : this.mode &= ~this.readMode;
    }
    get write() {
      return (this.mode & this.writeMode) === this.writeMode;
    }
    set write(val) {
      val ? this.mode |= this.writeMode : this.mode &= ~this.writeMode;
    }
    get isFolder() {
      return FS.isDir(this.mode);
    }
    get isDevice() {
      return FS.isChrdev(this.mode);
    }
  },
  lookupPath(path, opts = {}) {
    if (!path) {
      throw new FS.ErrnoError(44);
    }
    opts.follow_mount ??= true;
    if (!PATH.isAbs(path)) {
      path = FS.cwd() + "/" + path;
    }
    // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
    linkloop: for (var nlinks = 0; nlinks < 40; nlinks++) {
      // split the absolute path
      var parts = path.split("/").filter(p => !!p);
      // start at the root
      var current = FS.root;
      var current_path = "/";
      for (var i = 0; i < parts.length; i++) {
        var islast = (i === parts.length - 1);
        if (islast && opts.parent) {
          // stop resolving
          break;
        }
        if (parts[i] === ".") {
          continue;
        }
        if (parts[i] === "..") {
          current_path = PATH.dirname(current_path);
          if (FS.isRoot(current)) {
            path = current_path + "/" + parts.slice(i + 1).join("/");
            continue linkloop;
          } else {
            current = current.parent;
          }
          continue;
        }
        current_path = PATH.join2(current_path, parts[i]);
        try {
          current = FS.lookupNode(current, parts[i]);
        } catch (e) {
          // if noent_okay is true, suppress a ENOENT in the last component
          // and return an object with an undefined node. This is needed for
          // resolving symlinks in the path when creating a file.
          if ((e?.errno === 44) && islast && opts.noent_okay) {
            return {
              path: current_path
            };
          }
          throw e;
        }
        // jump to the mount's root node if this is a mountpoint
        if (FS.isMountpoint(current) && (!islast || opts.follow_mount)) {
          current = current.mounted.root;
        }
        // by default, lookupPath will not follow a symlink if it is the final path component.
        // setting opts.follow = true will override this behavior.
        if (FS.isLink(current.mode) && (!islast || opts.follow)) {
          if (!current.node_ops.readlink) {
            throw new FS.ErrnoError(52);
          }
          var link = current.node_ops.readlink(current);
          if (!PATH.isAbs(link)) {
            link = PATH.dirname(current_path) + "/" + link;
          }
          path = link + "/" + parts.slice(i + 1).join("/");
          continue linkloop;
        }
      }
      return {
        path: current_path,
        node: current
      };
    }
    throw new FS.ErrnoError(32);
  },
  getPath(node) {
    var path;
    while (true) {
      if (FS.isRoot(node)) {
        var mount = node.mount.mountpoint;
        if (!path) return mount;
        return mount[mount.length - 1] !== "/" ? `${mount}/${path}` : mount + path;
      }
      path = path ? `${node.name}/${path}` : node.name;
      node = node.parent;
    }
  },
  hashName(parentid, name) {
    var hash = 0;
    for (var i = 0; i < name.length; i++) {
      hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
    }
    return ((parentid + hash) >>> 0) % FS.nameTable.length;
  },
  hashAddNode(node) {
    var hash = FS.hashName(node.parent.id, node.name);
    node.name_next = FS.nameTable[hash];
    FS.nameTable[hash] = node;
  },
  hashRemoveNode(node) {
    var hash = FS.hashName(node.parent.id, node.name);
    if (FS.nameTable[hash] === node) {
      FS.nameTable[hash] = node.name_next;
    } else {
      var current = FS.nameTable[hash];
      while (current) {
        if (current.name_next === node) {
          current.name_next = node.name_next;
          break;
        }
        current = current.name_next;
      }
    }
  },
  lookupNode(parent, name) {
    var errCode = FS.mayLookup(parent);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    var hash = FS.hashName(parent.id, name);
    for (var node = FS.nameTable[hash]; node; node = node.name_next) {
      var nodeName = node.name;
      if (node.parent.id === parent.id && nodeName === name) {
        return node;
      }
    }
    // if we failed to find it in the cache, call into the VFS
    return FS.lookup(parent, name);
  },
  createNode(parent, name, mode, rdev) {
    var node = new FS.FSNode(parent, name, mode, rdev);
    FS.hashAddNode(node);
    return node;
  },
  destroyNode(node) {
    FS.hashRemoveNode(node);
  },
  isRoot(node) {
    return node === node.parent;
  },
  isMountpoint(node) {
    return !!node.mounted;
  },
  isFile(mode) {
    return (mode & 61440) === 32768;
  },
  isDir(mode) {
    return (mode & 61440) === 16384;
  },
  isLink(mode) {
    return (mode & 61440) === 40960;
  },
  isChrdev(mode) {
    return (mode & 61440) === 8192;
  },
  isBlkdev(mode) {
    return (mode & 61440) === 24576;
  },
  isFIFO(mode) {
    return (mode & 61440) === 4096;
  },
  isSocket(mode) {
    return (mode & 49152) === 49152;
  },
  flagsToPermissionString(flag) {
    var perms = [ "r", "w", "rw" ][flag & 3];
    if ((flag & 512)) {
      perms += "w";
    }
    return perms;
  },
  nodePermissions(node, perms) {
    if (FS.ignorePermissions) {
      return 0;
    }
    // return 0 if any user, group or owner bits are set.
    if (perms.includes("r") && !(node.mode & 292)) {
      return 2;
    } else if (perms.includes("w") && !(node.mode & 146)) {
      return 2;
    } else if (perms.includes("x") && !(node.mode & 73)) {
      return 2;
    }
    return 0;
  },
  mayLookup(dir) {
    if (!FS.isDir(dir.mode)) return 54;
    var errCode = FS.nodePermissions(dir, "x");
    if (errCode) return errCode;
    if (!dir.node_ops.lookup) return 2;
    return 0;
  },
  mayCreate(dir, name) {
    if (!FS.isDir(dir.mode)) {
      return 54;
    }
    try {
      var node = FS.lookupNode(dir, name);
      return 20;
    } catch (e) {}
    return FS.nodePermissions(dir, "wx");
  },
  mayDelete(dir, name, isdir) {
    var node;
    try {
      node = FS.lookupNode(dir, name);
    } catch (e) {
      return e.errno;
    }
    var errCode = FS.nodePermissions(dir, "wx");
    if (errCode) {
      return errCode;
    }
    if (isdir) {
      if (!FS.isDir(node.mode)) {
        return 54;
      }
      if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
        return 10;
      }
    } else {
      if (FS.isDir(node.mode)) {
        return 31;
      }
    }
    return 0;
  },
  mayOpen(node, flags) {
    if (!node) {
      return 44;
    }
    if (FS.isLink(node.mode)) {
      return 32;
    } else if (FS.isDir(node.mode)) {
      if (FS.flagsToPermissionString(flags) !== "r" || (flags & (512 | 64))) {
        // TODO: check for O_SEARCH? (== search for dir only)
        return 31;
      }
    }
    return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
  },
  checkOpExists(op, err) {
    if (!op) {
      throw new FS.ErrnoError(err);
    }
    return op;
  },
  MAX_OPEN_FDS: 4096,
  nextfd() {
    for (var fd = 0; fd <= FS.MAX_OPEN_FDS; fd++) {
      if (!FS.streams[fd]) {
        return fd;
      }
    }
    throw new FS.ErrnoError(33);
  },
  getStreamChecked(fd) {
    var stream = FS.getStream(fd);
    if (!stream) {
      throw new FS.ErrnoError(8);
    }
    return stream;
  },
  getStream: fd => FS.streams[fd],
  createStream(stream, fd = -1) {
    // clone it, so we can return an instance of FSStream
    stream = Object.assign(new FS.FSStream, stream);
    if (fd == -1) {
      fd = FS.nextfd();
    }
    stream.fd = fd;
    FS.streams[fd] = stream;
    return stream;
  },
  closeStream(fd) {
    FS.streams[fd] = null;
  },
  dupStream(origStream, fd = -1) {
    var stream = FS.createStream(origStream, fd);
    stream.stream_ops?.dup?.(stream);
    return stream;
  },
  doSetAttr(stream, node, attr) {
    var setattr = stream?.stream_ops.setattr;
    var arg = setattr ? stream : node;
    setattr ??= node.node_ops.setattr;
    FS.checkOpExists(setattr, 63);
    setattr(arg, attr);
  },
  chrdev_stream_ops: {
    open(stream) {
      var device = FS.getDevice(stream.node.rdev);
      // override node's stream ops with the device's
      stream.stream_ops = device.stream_ops;
      // forward the open call
      stream.stream_ops.open?.(stream);
    },
    llseek() {
      throw new FS.ErrnoError(70);
    }
  },
  major: dev => ((dev) >> 8),
  minor: dev => ((dev) & 255),
  makedev: (ma, mi) => ((ma) << 8 | (mi)),
  registerDevice(dev, ops) {
    FS.devices[dev] = {
      stream_ops: ops
    };
  },
  getDevice: dev => FS.devices[dev],
  getMounts(mount) {
    var mounts = [];
    var check = [ mount ];
    while (check.length) {
      var m = check.pop();
      mounts.push(m);
      check.push(...m.mounts);
    }
    return mounts;
  },
  syncfs(populate, callback) {
    if (typeof populate == "function") {
      callback = populate;
      populate = false;
    }
    FS.syncFSRequests++;
    if (FS.syncFSRequests > 1) {
      err(`warning: ${FS.syncFSRequests} FS.syncfs operations in flight at once, probably just doing extra work`);
    }
    var mounts = FS.getMounts(FS.root.mount);
    var completed = 0;
    function doCallback(errCode) {
      FS.syncFSRequests--;
      return callback(errCode);
    }
    function done(errCode) {
      if (errCode) {
        if (!done.errored) {
          done.errored = true;
          return doCallback(errCode);
        }
        return;
      }
      if (++completed >= mounts.length) {
        doCallback(null);
      }
    }
    // sync all mounts
    mounts.forEach(mount => {
      if (!mount.type.syncfs) {
        return done(null);
      }
      mount.type.syncfs(mount, populate, done);
    });
  },
  mount(type, opts, mountpoint) {
    var root = mountpoint === "/";
    var pseudo = !mountpoint;
    var node;
    if (root && FS.root) {
      throw new FS.ErrnoError(10);
    } else if (!root && !pseudo) {
      var lookup = FS.lookupPath(mountpoint, {
        follow_mount: false
      });
      mountpoint = lookup.path;
      // use the absolute path
      node = lookup.node;
      if (FS.isMountpoint(node)) {
        throw new FS.ErrnoError(10);
      }
      if (!FS.isDir(node.mode)) {
        throw new FS.ErrnoError(54);
      }
    }
    var mount = {
      type,
      opts,
      mountpoint,
      mounts: []
    };
    // create a root node for the fs
    var mountRoot = type.mount(mount);
    mountRoot.mount = mount;
    mount.root = mountRoot;
    if (root) {
      FS.root = mountRoot;
    } else if (node) {
      // set as a mountpoint
      node.mounted = mount;
      // add the new mount to the current mount's children
      if (node.mount) {
        node.mount.mounts.push(mount);
      }
    }
    return mountRoot;
  },
  unmount(mountpoint) {
    var lookup = FS.lookupPath(mountpoint, {
      follow_mount: false
    });
    if (!FS.isMountpoint(lookup.node)) {
      throw new FS.ErrnoError(28);
    }
    // destroy the nodes for this mount, and all its child mounts
    var node = lookup.node;
    var mount = node.mounted;
    var mounts = FS.getMounts(mount);
    Object.keys(FS.nameTable).forEach(hash => {
      var current = FS.nameTable[hash];
      while (current) {
        var next = current.name_next;
        if (mounts.includes(current.mount)) {
          FS.destroyNode(current);
        }
        current = next;
      }
    });
    // no longer a mountpoint
    node.mounted = null;
    // remove this mount from the child mounts
    var idx = node.mount.mounts.indexOf(mount);
    node.mount.mounts.splice(idx, 1);
  },
  lookup(parent, name) {
    return parent.node_ops.lookup(parent, name);
  },
  mknod(path, mode, dev) {
    var lookup = FS.lookupPath(path, {
      parent: true
    });
    var parent = lookup.node;
    var name = PATH.basename(path);
    if (!name) {
      throw new FS.ErrnoError(28);
    }
    if (name === "." || name === "..") {
      throw new FS.ErrnoError(20);
    }
    var errCode = FS.mayCreate(parent, name);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    if (!parent.node_ops.mknod) {
      throw new FS.ErrnoError(63);
    }
    return parent.node_ops.mknod(parent, name, mode, dev);
  },
  statfs(path) {
    return FS.statfsNode(FS.lookupPath(path, {
      follow: true
    }).node);
  },
  statfsStream(stream) {
    // We keep a separate statfsStream function because noderawfs overrides
    // it. In noderawfs, stream.node is sometimes null. Instead, we need to
    // look at stream.path.
    return FS.statfsNode(stream.node);
  },
  statfsNode(node) {
    // NOTE: None of the defaults here are true. We're just returning safe and
    //       sane values. Currently nodefs and rawfs replace these defaults,
    //       other file systems leave them alone.
    var rtn = {
      bsize: 4096,
      frsize: 4096,
      blocks: 1e6,
      bfree: 5e5,
      bavail: 5e5,
      files: FS.nextInode,
      ffree: FS.nextInode - 1,
      fsid: 42,
      flags: 2,
      namelen: 255
    };
    if (node.node_ops.statfs) {
      Object.assign(rtn, node.node_ops.statfs(node.mount.opts.root));
    }
    return rtn;
  },
  create(path, mode = 438) {
    mode &= 4095;
    mode |= 32768;
    return FS.mknod(path, mode, 0);
  },
  mkdir(path, mode = 511) {
    mode &= 511 | 512;
    mode |= 16384;
    return FS.mknod(path, mode, 0);
  },
  mkdirTree(path, mode) {
    var dirs = path.split("/");
    var d = "";
    for (var dir of dirs) {
      if (!dir) continue;
      if (d || PATH.isAbs(path)) d += "/";
      d += dir;
      try {
        FS.mkdir(d, mode);
      } catch (e) {
        if (e.errno != 20) throw e;
      }
    }
  },
  mkdev(path, mode, dev) {
    if (typeof dev == "undefined") {
      dev = mode;
      mode = 438;
    }
    mode |= 8192;
    return FS.mknod(path, mode, dev);
  },
  symlink(oldpath, newpath) {
    if (!PATH_FS.resolve(oldpath)) {
      throw new FS.ErrnoError(44);
    }
    var lookup = FS.lookupPath(newpath, {
      parent: true
    });
    var parent = lookup.node;
    if (!parent) {
      throw new FS.ErrnoError(44);
    }
    var newname = PATH.basename(newpath);
    var errCode = FS.mayCreate(parent, newname);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    if (!parent.node_ops.symlink) {
      throw new FS.ErrnoError(63);
    }
    return parent.node_ops.symlink(parent, newname, oldpath);
  },
  rename(old_path, new_path) {
    var old_dirname = PATH.dirname(old_path);
    var new_dirname = PATH.dirname(new_path);
    var old_name = PATH.basename(old_path);
    var new_name = PATH.basename(new_path);
    // parents must exist
    var lookup, old_dir, new_dir;
    // let the errors from non existent directories percolate up
    lookup = FS.lookupPath(old_path, {
      parent: true
    });
    old_dir = lookup.node;
    lookup = FS.lookupPath(new_path, {
      parent: true
    });
    new_dir = lookup.node;
    if (!old_dir || !new_dir) throw new FS.ErrnoError(44);
    // need to be part of the same mount
    if (old_dir.mount !== new_dir.mount) {
      throw new FS.ErrnoError(75);
    }
    // source must exist
    var old_node = FS.lookupNode(old_dir, old_name);
    // old path should not be an ancestor of the new path
    var relative = PATH_FS.relative(old_path, new_dirname);
    if (relative.charAt(0) !== ".") {
      throw new FS.ErrnoError(28);
    }
    // new path should not be an ancestor of the old path
    relative = PATH_FS.relative(new_path, old_dirname);
    if (relative.charAt(0) !== ".") {
      throw new FS.ErrnoError(55);
    }
    // see if the new path already exists
    var new_node;
    try {
      new_node = FS.lookupNode(new_dir, new_name);
    } catch (e) {}
    // early out if nothing needs to change
    if (old_node === new_node) {
      return;
    }
    // we'll need to delete the old entry
    var isdir = FS.isDir(old_node.mode);
    var errCode = FS.mayDelete(old_dir, old_name, isdir);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    // need delete permissions if we'll be overwriting.
    // need create permissions if new doesn't already exist.
    errCode = new_node ? FS.mayDelete(new_dir, new_name, isdir) : FS.mayCreate(new_dir, new_name);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    if (!old_dir.node_ops.rename) {
      throw new FS.ErrnoError(63);
    }
    if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
      throw new FS.ErrnoError(10);
    }
    // if we are going to change the parent, check write permissions
    if (new_dir !== old_dir) {
      errCode = FS.nodePermissions(old_dir, "w");
      if (errCode) {
        throw new FS.ErrnoError(errCode);
      }
    }
    // remove the node from the lookup hash
    FS.hashRemoveNode(old_node);
    // do the underlying fs rename
    try {
      old_dir.node_ops.rename(old_node, new_dir, new_name);
      // update old node (we do this here to avoid each backend
      // needing to)
      old_node.parent = new_dir;
    } catch (e) {
      throw e;
    } finally {
      // add the node back to the hash (in case node_ops.rename
      // changed its name)
      FS.hashAddNode(old_node);
    }
  },
  rmdir(path) {
    var lookup = FS.lookupPath(path, {
      parent: true
    });
    var parent = lookup.node;
    var name = PATH.basename(path);
    var node = FS.lookupNode(parent, name);
    var errCode = FS.mayDelete(parent, name, true);
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    if (!parent.node_ops.rmdir) {
      throw new FS.ErrnoError(63);
    }
    if (FS.isMountpoint(node)) {
      throw new FS.ErrnoError(10);
    }
    parent.node_ops.rmdir(parent, name);
    FS.destroyNode(node);
  },
  readdir(path) {
    var lookup = FS.lookupPath(path, {
      follow: true
    });
    var node = lookup.node;
    var readdir = FS.checkOpExists(node.node_ops.readdir, 54);
    return readdir(node);
  },
  unlink(path) {
    var lookup = FS.lookupPath(path, {
      parent: true
    });
    var parent = lookup.node;
    if (!parent) {
      throw new FS.ErrnoError(44);
    }
    var name = PATH.basename(path);
    var node = FS.lookupNode(parent, name);
    var errCode = FS.mayDelete(parent, name, false);
    if (errCode) {
      // According to POSIX, we should map EISDIR to EPERM, but
      // we instead do what Linux does (and we must, as we use
      // the musl linux libc).
      throw new FS.ErrnoError(errCode);
    }
    if (!parent.node_ops.unlink) {
      throw new FS.ErrnoError(63);
    }
    if (FS.isMountpoint(node)) {
      throw new FS.ErrnoError(10);
    }
    parent.node_ops.unlink(parent, name);
    FS.destroyNode(node);
  },
  readlink(path) {
    var lookup = FS.lookupPath(path);
    var link = lookup.node;
    if (!link) {
      throw new FS.ErrnoError(44);
    }
    if (!link.node_ops.readlink) {
      throw new FS.ErrnoError(28);
    }
    return link.node_ops.readlink(link);
  },
  stat(path, dontFollow) {
    var lookup = FS.lookupPath(path, {
      follow: !dontFollow
    });
    var node = lookup.node;
    var getattr = FS.checkOpExists(node.node_ops.getattr, 63);
    return getattr(node);
  },
  fstat(fd) {
    var stream = FS.getStreamChecked(fd);
    var node = stream.node;
    var getattr = stream.stream_ops.getattr;
    var arg = getattr ? stream : node;
    getattr ??= node.node_ops.getattr;
    FS.checkOpExists(getattr, 63);
    return getattr(arg);
  },
  lstat(path) {
    return FS.stat(path, true);
  },
  doChmod(stream, node, mode, dontFollow) {
    FS.doSetAttr(stream, node, {
      mode: (mode & 4095) | (node.mode & ~4095),
      ctime: Date.now(),
      dontFollow
    });
  },
  chmod(path, mode, dontFollow) {
    var node;
    if (typeof path == "string") {
      var lookup = FS.lookupPath(path, {
        follow: !dontFollow
      });
      node = lookup.node;
    } else {
      node = path;
    }
    FS.doChmod(null, node, mode, dontFollow);
  },
  lchmod(path, mode) {
    FS.chmod(path, mode, true);
  },
  fchmod(fd, mode) {
    var stream = FS.getStreamChecked(fd);
    FS.doChmod(stream, stream.node, mode, false);
  },
  doChown(stream, node, dontFollow) {
    FS.doSetAttr(stream, node, {
      timestamp: Date.now(),
      dontFollow
    });
  },
  chown(path, uid, gid, dontFollow) {
    var node;
    if (typeof path == "string") {
      var lookup = FS.lookupPath(path, {
        follow: !dontFollow
      });
      node = lookup.node;
    } else {
      node = path;
    }
    FS.doChown(null, node, dontFollow);
  },
  lchown(path, uid, gid) {
    FS.chown(path, uid, gid, true);
  },
  fchown(fd, uid, gid) {
    var stream = FS.getStreamChecked(fd);
    FS.doChown(stream, stream.node, false);
  },
  doTruncate(stream, node, len) {
    if (FS.isDir(node.mode)) {
      throw new FS.ErrnoError(31);
    }
    if (!FS.isFile(node.mode)) {
      throw new FS.ErrnoError(28);
    }
    var errCode = FS.nodePermissions(node, "w");
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    FS.doSetAttr(stream, node, {
      size: len,
      timestamp: Date.now()
    });
  },
  truncate(path, len) {
    if (len < 0) {
      throw new FS.ErrnoError(28);
    }
    var node;
    if (typeof path == "string") {
      var lookup = FS.lookupPath(path, {
        follow: true
      });
      node = lookup.node;
    } else {
      node = path;
    }
    FS.doTruncate(null, node, len);
  },
  ftruncate(fd, len) {
    var stream = FS.getStreamChecked(fd);
    if (len < 0 || (stream.flags & 2097155) === 0) {
      throw new FS.ErrnoError(28);
    }
    FS.doTruncate(stream, stream.node, len);
  },
  utime(path, atime, mtime) {
    var lookup = FS.lookupPath(path, {
      follow: true
    });
    var node = lookup.node;
    var setattr = FS.checkOpExists(node.node_ops.setattr, 63);
    setattr(node, {
      atime,
      mtime
    });
  },
  open(path, flags, mode = 438) {
    if (path === "") {
      throw new FS.ErrnoError(44);
    }
    flags = typeof flags == "string" ? FS_modeStringToFlags(flags) : flags;
    if ((flags & 64)) {
      mode = (mode & 4095) | 32768;
    } else {
      mode = 0;
    }
    var node;
    var isDirPath;
    if (typeof path == "object") {
      node = path;
    } else {
      isDirPath = path.endsWith("/");
      // noent_okay makes it so that if the final component of the path
      // doesn't exist, lookupPath returns `node: undefined`. `path` will be
      // updated to point to the target of all symlinks.
      var lookup = FS.lookupPath(path, {
        follow: !(flags & 131072),
        noent_okay: true
      });
      node = lookup.node;
      path = lookup.path;
    }
    // perhaps we need to create the node
    var created = false;
    if ((flags & 64)) {
      if (node) {
        // if O_CREAT and O_EXCL are set, error out if the node already exists
        if ((flags & 128)) {
          throw new FS.ErrnoError(20);
        }
      } else if (isDirPath) {
        throw new FS.ErrnoError(31);
      } else {
        // node doesn't exist, try to create it
        // Ignore the permission bits here to ensure we can `open` this new
        // file below. We use chmod below the apply the permissions once the
        // file is open.
        node = FS.mknod(path, mode | 511, 0);
        created = true;
      }
    }
    if (!node) {
      throw new FS.ErrnoError(44);
    }
    // can't truncate a device
    if (FS.isChrdev(node.mode)) {
      flags &= ~512;
    }
    // if asked only for a directory, then this must be one
    if ((flags & 65536) && !FS.isDir(node.mode)) {
      throw new FS.ErrnoError(54);
    }
    // check permissions, if this is not a file we just created now (it is ok to
    // create and write to a file with read-only permissions; it is read-only
    // for later use)
    if (!created) {
      var errCode = FS.mayOpen(node, flags);
      if (errCode) {
        throw new FS.ErrnoError(errCode);
      }
    }
    // do truncation if necessary
    if ((flags & 512) && !created) {
      FS.truncate(node, 0);
    }
    // we've already handled these, don't pass down to the underlying vfs
    flags &= ~(128 | 512 | 131072);
    // register the stream with the filesystem
    var stream = FS.createStream({
      node,
      path: FS.getPath(node),
      // we want the absolute path to the node
      flags,
      seekable: true,
      position: 0,
      stream_ops: node.stream_ops,
      // used by the file family libc calls (fopen, fwrite, ferror, etc.)
      ungotten: [],
      error: false
    });
    // call the new stream's open function
    if (stream.stream_ops.open) {
      stream.stream_ops.open(stream);
    }
    if (created) {
      FS.chmod(node, mode & 511);
    }
    if (Module["logReadFiles"] && !(flags & 1)) {
      if (!(path in FS.readFiles)) {
        FS.readFiles[path] = 1;
      }
    }
    return stream;
  },
  close(stream) {
    if (FS.isClosed(stream)) {
      throw new FS.ErrnoError(8);
    }
    if (stream.getdents) stream.getdents = null;
    // free readdir state
    try {
      if (stream.stream_ops.close) {
        stream.stream_ops.close(stream);
      }
    } catch (e) {
      throw e;
    } finally {
      FS.closeStream(stream.fd);
    }
    stream.fd = null;
  },
  isClosed(stream) {
    return stream.fd === null;
  },
  llseek(stream, offset, whence) {
    if (FS.isClosed(stream)) {
      throw new FS.ErrnoError(8);
    }
    if (!stream.seekable || !stream.stream_ops.llseek) {
      throw new FS.ErrnoError(70);
    }
    if (whence != 0 && whence != 1 && whence != 2) {
      throw new FS.ErrnoError(28);
    }
    stream.position = stream.stream_ops.llseek(stream, offset, whence);
    stream.ungotten = [];
    return stream.position;
  },
  read(stream, buffer, offset, length, position) {
    if (length < 0 || position < 0) {
      throw new FS.ErrnoError(28);
    }
    if (FS.isClosed(stream)) {
      throw new FS.ErrnoError(8);
    }
    if ((stream.flags & 2097155) === 1) {
      throw new FS.ErrnoError(8);
    }
    if (FS.isDir(stream.node.mode)) {
      throw new FS.ErrnoError(31);
    }
    if (!stream.stream_ops.read) {
      throw new FS.ErrnoError(28);
    }
    var seeking = typeof position != "undefined";
    if (!seeking) {
      position = stream.position;
    } else if (!stream.seekable) {
      throw new FS.ErrnoError(70);
    }
    var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
    if (!seeking) stream.position += bytesRead;
    return bytesRead;
  },
  write(stream, buffer, offset, length, position, canOwn) {
    if (length < 0 || position < 0) {
      throw new FS.ErrnoError(28);
    }
    if (FS.isClosed(stream)) {
      throw new FS.ErrnoError(8);
    }
    if ((stream.flags & 2097155) === 0) {
      throw new FS.ErrnoError(8);
    }
    if (FS.isDir(stream.node.mode)) {
      throw new FS.ErrnoError(31);
    }
    if (!stream.stream_ops.write) {
      throw new FS.ErrnoError(28);
    }
    if (stream.seekable && stream.flags & 1024) {
      // seek to the end before writing in append mode
      FS.llseek(stream, 0, 2);
    }
    var seeking = typeof position != "undefined";
    if (!seeking) {
      position = stream.position;
    } else if (!stream.seekable) {
      throw new FS.ErrnoError(70);
    }
    var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
    if (!seeking) stream.position += bytesWritten;
    return bytesWritten;
  },
  mmap(stream, length, position, prot, flags) {
    // User requests writing to file (prot & PROT_WRITE != 0).
    // Checking if we have permissions to write to the file unless
    // MAP_PRIVATE flag is set. According to POSIX spec it is possible
    // to write to file opened in read-only mode with MAP_PRIVATE flag,
    // as all modifications will be visible only in the memory of
    // the current process.
    if ((prot & 2) !== 0 && (flags & 2) === 0 && (stream.flags & 2097155) !== 2) {
      throw new FS.ErrnoError(2);
    }
    if ((stream.flags & 2097155) === 1) {
      throw new FS.ErrnoError(2);
    }
    if (!stream.stream_ops.mmap) {
      throw new FS.ErrnoError(43);
    }
    if (!length) {
      throw new FS.ErrnoError(28);
    }
    return stream.stream_ops.mmap(stream, length, position, prot, flags);
  },
  msync(stream, buffer, offset, length, mmapFlags) {
    if (!stream.stream_ops.msync) {
      return 0;
    }
    return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
  },
  ioctl(stream, cmd, arg) {
    if (!stream.stream_ops.ioctl) {
      throw new FS.ErrnoError(59);
    }
    return stream.stream_ops.ioctl(stream, cmd, arg);
  },
  readFile(path, opts = {}) {
    opts.flags = opts.flags || 0;
    opts.encoding = opts.encoding || "binary";
    if (opts.encoding !== "utf8" && opts.encoding !== "binary") {
      throw new Error(`Invalid encoding type "${opts.encoding}"`);
    }
    var stream = FS.open(path, opts.flags);
    var stat = FS.stat(path);
    var length = stat.size;
    var buf = new Uint8Array(length);
    FS.read(stream, buf, 0, length, 0);
    if (opts.encoding === "utf8") {
      buf = UTF8ArrayToString(buf);
    }
    FS.close(stream);
    return buf;
  },
  writeFile(path, data, opts = {}) {
    opts.flags = opts.flags || 577;
    var stream = FS.open(path, opts.flags, opts.mode);
    if (typeof data == "string") {
      data = new Uint8Array(intArrayFromString(data, true));
    }
    if (ArrayBuffer.isView(data)) {
      FS.write(stream, data, 0, data.byteLength, undefined, opts.canOwn);
    } else {
      throw new Error("Unsupported data type");
    }
    FS.close(stream);
  },
  cwd: () => FS.currentPath,
  chdir(path) {
    var lookup = FS.lookupPath(path, {
      follow: true
    });
    if (lookup.node === null) {
      throw new FS.ErrnoError(44);
    }
    if (!FS.isDir(lookup.node.mode)) {
      throw new FS.ErrnoError(54);
    }
    var errCode = FS.nodePermissions(lookup.node, "x");
    if (errCode) {
      throw new FS.ErrnoError(errCode);
    }
    FS.currentPath = lookup.path;
  },
  createDefaultDirectories() {
    FS.mkdir("/tmp");
    FS.mkdir("/home");
    FS.mkdir("/home/web_user");
  },
  createDefaultDevices() {
    // create /dev
    FS.mkdir("/dev");
    // setup /dev/null
    FS.registerDevice(FS.makedev(1, 3), {
      read: () => 0,
      write: (stream, buffer, offset, length, pos) => length,
      llseek: () => 0
    });
    FS.mkdev("/dev/null", FS.makedev(1, 3));
    // setup /dev/tty and /dev/tty1
    // stderr needs to print output using err() rather than out()
    // so we register a second tty just for it.
    TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
    TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
    FS.mkdev("/dev/tty", FS.makedev(5, 0));
    FS.mkdev("/dev/tty1", FS.makedev(6, 0));
    // setup /dev/[u]random
    // use a buffer to avoid overhead of individual crypto calls per byte
    var randomBuffer = new Uint8Array(1024), randomLeft = 0;
    var randomByte = () => {
      if (randomLeft === 0) {
        randomFill(randomBuffer);
        randomLeft = randomBuffer.byteLength;
      }
      return randomBuffer[--randomLeft];
    };
    FS.createDevice("/dev", "random", randomByte);
    FS.createDevice("/dev", "urandom", randomByte);
    // we're not going to emulate the actual shm device,
    // just create the tmp dirs that reside in it commonly
    FS.mkdir("/dev/shm");
    FS.mkdir("/dev/shm/tmp");
  },
  createSpecialDirectories() {
    // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the
    // name of the stream for fd 6 (see test_unistd_ttyname)
    FS.mkdir("/proc");
    var proc_self = FS.mkdir("/proc/self");
    FS.mkdir("/proc/self/fd");
    FS.mount({
      mount() {
        var node = FS.createNode(proc_self, "fd", 16895, 73);
        node.stream_ops = {
          llseek: MEMFS.stream_ops.llseek
        };
        node.node_ops = {
          lookup(parent, name) {
            var fd = +name;
            var stream = FS.getStreamChecked(fd);
            var ret = {
              parent: null,
              mount: {
                mountpoint: "fake"
              },
              node_ops: {
                readlink: () => stream.path
              },
              id: fd + 1
            };
            ret.parent = ret;
            // make it look like a simple root node
            return ret;
          },
          readdir() {
            return Array.from(FS.streams.entries()).filter(([k, v]) => v).map(([k, v]) => k.toString());
          }
        };
        return node;
      }
    }, {}, "/proc/self/fd");
  },
  createStandardStreams(input, output, error) {
    // TODO deprecate the old functionality of a single
    // input / output callback and that utilizes FS.createDevice
    // and instead require a unique set of stream ops
    // by default, we symlink the standard streams to the
    // default tty devices. however, if the standard streams
    // have been overwritten we create a unique device for
    // them instead.
    if (input) {
      FS.createDevice("/dev", "stdin", input);
    } else {
      FS.symlink("/dev/tty", "/dev/stdin");
    }
    if (output) {
      FS.createDevice("/dev", "stdout", null, output);
    } else {
      FS.symlink("/dev/tty", "/dev/stdout");
    }
    if (error) {
      FS.createDevice("/dev", "stderr", null, error);
    } else {
      FS.symlink("/dev/tty1", "/dev/stderr");
    }
    // open default streams for the stdin, stdout and stderr devices
    var stdin = FS.open("/dev/stdin", 0);
    var stdout = FS.open("/dev/stdout", 1);
    var stderr = FS.open("/dev/stderr", 1);
  },
  staticInit() {
    FS.nameTable = new Array(4096);
    FS.mount(MEMFS, {}, "/");
    FS.createDefaultDirectories();
    FS.createDefaultDevices();
    FS.createSpecialDirectories();
    FS.filesystems = {
      "MEMFS": MEMFS
    };
  },
  init(input, output, error) {
    FS.initialized = true;
    // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
    input ??= Module["stdin"];
    output ??= Module["stdout"];
    error ??= Module["stderr"];
    FS.createStandardStreams(input, output, error);
  },
  quit() {
    FS.initialized = false;
    // force-flush all streams, so we get musl std streams printed out
    // close all of our streams
    for (var stream of FS.streams) {
      if (stream) {
        FS.close(stream);
      }
    }
  },
  findObject(path, dontResolveLastLink) {
    var ret = FS.analyzePath(path, dontResolveLastLink);
    if (!ret.exists) {
      return null;
    }
    return ret.object;
  },
  analyzePath(path, dontResolveLastLink) {
    // operate from within the context of the symlink's target
    try {
      var lookup = FS.lookupPath(path, {
        follow: !dontResolveLastLink
      });
      path = lookup.path;
    } catch (e) {}
    var ret = {
      isRoot: false,
      exists: false,
      error: 0,
      name: null,
      path: null,
      object: null,
      parentExists: false,
      parentPath: null,
      parentObject: null
    };
    try {
      var lookup = FS.lookupPath(path, {
        parent: true
      });
      ret.parentExists = true;
      ret.parentPath = lookup.path;
      ret.parentObject = lookup.node;
      ret.name = PATH.basename(path);
      lookup = FS.lookupPath(path, {
        follow: !dontResolveLastLink
      });
      ret.exists = true;
      ret.path = lookup.path;
      ret.object = lookup.node;
      ret.name = lookup.node.name;
      ret.isRoot = lookup.path === "/";
    } catch (e) {
      ret.error = e.errno;
    }
    return ret;
  },
  createPath(parent, path, canRead, canWrite) {
    parent = typeof parent == "string" ? parent : FS.getPath(parent);
    var parts = path.split("/").reverse();
    while (parts.length) {
      var part = parts.pop();
      if (!part) continue;
      var current = PATH.join2(parent, part);
      try {
        FS.mkdir(current);
      } catch (e) {
        if (e.errno != 20) throw e;
      }
      parent = current;
    }
    return current;
  },
  createFile(parent, name, properties, canRead, canWrite) {
    var path = PATH.join2(typeof parent == "string" ? parent : FS.getPath(parent), name);
    var mode = FS_getMode(canRead, canWrite);
    return FS.create(path, mode);
  },
  createDataFile(parent, name, data, canRead, canWrite, canOwn) {
    var path = name;
    if (parent) {
      parent = typeof parent == "string" ? parent : FS.getPath(parent);
      path = name ? PATH.join2(parent, name) : parent;
    }
    var mode = FS_getMode(canRead, canWrite);
    var node = FS.create(path, mode);
    if (data) {
      if (typeof data == "string") {
        var arr = new Array(data.length);
        for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
        data = arr;
      }
      // make sure we can write to the file
      FS.chmod(node, mode | 146);
      var stream = FS.open(node, 577);
      FS.write(stream, data, 0, data.length, 0, canOwn);
      FS.close(stream);
      FS.chmod(node, mode);
    }
  },
  createDevice(parent, name, input, output) {
    var path = PATH.join2(typeof parent == "string" ? parent : FS.getPath(parent), name);
    var mode = FS_getMode(!!input, !!output);
    FS.createDevice.major ??= 64;
    var dev = FS.makedev(FS.createDevice.major++, 0);
    // Create a fake device that a set of stream ops to emulate
    // the old behavior.
    FS.registerDevice(dev, {
      open(stream) {
        stream.seekable = false;
      },
      close(stream) {
        // flush any pending line data
        if (output?.buffer?.length) {
          output(10);
        }
      },
      read(stream, buffer, offset, length, pos) {
        var bytesRead = 0;
        for (var i = 0; i < length; i++) {
          var result;
          try {
            result = input();
          } catch (e) {
            throw new FS.ErrnoError(29);
          }
          if (result === undefined && bytesRead === 0) {
            throw new FS.ErrnoError(6);
          }
          if (result === null || result === undefined) break;
          bytesRead++;
          buffer[offset + i] = result;
        }
        if (bytesRead) {
          stream.node.atime = Date.now();
        }
        return bytesRead;
      },
      write(stream, buffer, offset, length, pos) {
        for (var i = 0; i < length; i++) {
          try {
            output(buffer[offset + i]);
          } catch (e) {
            throw new FS.ErrnoError(29);
          }
        }
        if (length) {
          stream.node.mtime = stream.node.ctime = Date.now();
        }
        return i;
      }
    });
    return FS.mkdev(path, mode, dev);
  },
  forceLoadFile(obj) {
    if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
    if (typeof XMLHttpRequest != "undefined") {
      throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
    } else {
      // Command-line.
      try {
        obj.contents = readBinary(obj.url);
        obj.usedBytes = obj.contents.length;
      } catch (e) {
        throw new FS.ErrnoError(29);
      }
    }
  },
  createLazyFile(parent, name, url, canRead, canWrite) {
    // Lazy chunked Uint8Array (implements get and length from Uint8Array).
    // Actual getting is abstracted away for eventual reuse.
    class LazyUint8Array {
      lengthKnown=false;
      chunks=[];
      // Loaded chunks. Index is the chunk number
      get(idx) {
        if (idx > this.length - 1 || idx < 0) {
          return undefined;
        }
        var chunkOffset = idx % this.chunkSize;
        var chunkNum = (idx / this.chunkSize) | 0;
        return this.getter(chunkNum)[chunkOffset];
      }
      setDataGetter(getter) {
        this.getter = getter;
      }
      cacheLength() {
        // Find length
        var xhr = new XMLHttpRequest;
        xhr.open("HEAD", url, false);
        xhr.send(null);
        if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
        var datalength = Number(xhr.getResponseHeader("Content-length"));
        var header;
        var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
        var usesGzip = (header = xhr.getResponseHeader("Content-Encoding")) && header === "gzip";
        var chunkSize = 1024 * 1024;
        // Chunk size in bytes
        if (!hasByteServing) chunkSize = datalength;
        // Function to get a range from the remote URL.
        var doXHR = (from, to) => {
          if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
          if (to > datalength - 1) throw new Error("only " + datalength + " bytes available! programmer error!");
          // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
          var xhr = new XMLHttpRequest;
          xhr.open("GET", url, false);
          if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
          // Some hints to the browser that we want binary data.
          xhr.responseType = "arraybuffer";
          if (xhr.overrideMimeType) {
            xhr.overrideMimeType("text/plain; charset=x-user-defined");
          }
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          if (xhr.response !== undefined) {
            return new Uint8Array(/** @type{Array<number>} */ (xhr.response || []));
          }
          return intArrayFromString(xhr.responseText || "", true);
        };
        var lazyArray = this;
        lazyArray.setDataGetter(chunkNum => {
          var start = chunkNum * chunkSize;
          var end = (chunkNum + 1) * chunkSize - 1;
          // including this byte
          end = Math.min(end, datalength - 1);
          // if datalength-1 is selected, this is the last block
          if (typeof lazyArray.chunks[chunkNum] == "undefined") {
            lazyArray.chunks[chunkNum] = doXHR(start, end);
          }
          if (typeof lazyArray.chunks[chunkNum] == "undefined") throw new Error("doXHR failed!");
          return lazyArray.chunks[chunkNum];
        });
        if (usesGzip || !datalength) {
          // if the server uses gzip or doesn't supply the length, we have to download the whole file to get the (uncompressed) length
          chunkSize = datalength = 1;
          // this will force getter(0)/doXHR do download the whole file
          datalength = this.getter(0).length;
          chunkSize = datalength;
          out("LazyFiles on gzip forces download of the whole file when length is accessed");
        }
        this._length = datalength;
        this._chunkSize = chunkSize;
        this.lengthKnown = true;
      }
      get length() {
        if (!this.lengthKnown) {
          this.cacheLength();
        }
        return this._length;
      }
      get chunkSize() {
        if (!this.lengthKnown) {
          this.cacheLength();
        }
        return this._chunkSize;
      }
    }
    if (typeof XMLHttpRequest != "undefined") {
      if (!ENVIRONMENT_IS_WORKER) throw "Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc";
      var lazyArray = new LazyUint8Array;
      var properties = {
        isDevice: false,
        contents: lazyArray
      };
    } else {
      var properties = {
        isDevice: false,
        url
      };
    }
    var node = FS.createFile(parent, name, properties, canRead, canWrite);
    // This is a total hack, but I want to get this lazy file code out of the
    // core of MEMFS. If we want to keep this lazy file concept I feel it should
    // be its own thin LAZYFS proxying calls to MEMFS.
    if (properties.contents) {
      node.contents = properties.contents;
    } else if (properties.url) {
      node.contents = null;
      node.url = properties.url;
    }
    // Add a function that defers querying the file size until it is asked the first time.
    Object.defineProperties(node, {
      usedBytes: {
        get: function() {
          return this.contents.length;
        }
      }
    });
    // override each stream op with one that tries to force load the lazy file first
    var stream_ops = {};
    var keys = Object.keys(node.stream_ops);
    keys.forEach(key => {
      var fn = node.stream_ops[key];
      stream_ops[key] = (...args) => {
        FS.forceLoadFile(node);
        return fn(...args);
      };
    });
    function writeChunks(stream, buffer, offset, length, position) {
      var contents = stream.node.contents;
      if (position >= contents.length) return 0;
      var size = Math.min(contents.length - position, length);
      if (contents.slice) {
        // normal array
        for (var i = 0; i < size; i++) {
          buffer[offset + i] = contents[position + i];
        }
      } else {
        for (var i = 0; i < size; i++) {
          // LazyUint8Array from sync binary XHR
          buffer[offset + i] = contents.get(position + i);
        }
      }
      return size;
    }
    // use a custom read function
    stream_ops.read = (stream, buffer, offset, length, position) => {
      FS.forceLoadFile(node);
      return writeChunks(stream, buffer, offset, length, position);
    };
    // use a custom mmap function
    stream_ops.mmap = (stream, length, position, prot, flags) => {
      FS.forceLoadFile(node);
      var ptr = mmapAlloc(length);
      if (!ptr) {
        throw new FS.ErrnoError(48);
      }
      writeChunks(stream, HEAP8, ptr, length, position);
      return {
        ptr,
        allocated: true
      };
    };
    node.stream_ops = stream_ops;
    return node;
  }
};

var SYSCALLS = {
  DEFAULT_POLLMASK: 5,
  calculateAt(dirfd, path, allowEmpty) {
    if (PATH.isAbs(path)) {
      return path;
    }
    // relative path
    var dir;
    if (dirfd === -100) {
      dir = FS.cwd();
    } else {
      var dirstream = SYSCALLS.getStreamFromFD(dirfd);
      dir = dirstream.path;
    }
    if (path.length == 0) {
      if (!allowEmpty) {
        throw new FS.ErrnoError(44);
      }
      return dir;
    }
    return dir + "/" + path;
  },
  writeStat(buf, stat) {
    HEAP32[((buf) >> 2)] = stat.dev;
    HEAP32[(((buf) + (4)) >> 2)] = stat.mode;
    HEAPU32[(((buf) + (8)) >> 2)] = stat.nlink;
    HEAP32[(((buf) + (12)) >> 2)] = stat.uid;
    HEAP32[(((buf) + (16)) >> 2)] = stat.gid;
    HEAP32[(((buf) + (20)) >> 2)] = stat.rdev;
    HEAP64[(((buf) + (24)) >> 3)] = BigInt(stat.size);
    HEAP32[(((buf) + (32)) >> 2)] = 4096;
    HEAP32[(((buf) + (36)) >> 2)] = stat.blocks;
    var atime = stat.atime.getTime();
    var mtime = stat.mtime.getTime();
    var ctime = stat.ctime.getTime();
    HEAP64[(((buf) + (40)) >> 3)] = BigInt(Math.floor(atime / 1e3));
    HEAPU32[(((buf) + (48)) >> 2)] = (atime % 1e3) * 1e3 * 1e3;
    HEAP64[(((buf) + (56)) >> 3)] = BigInt(Math.floor(mtime / 1e3));
    HEAPU32[(((buf) + (64)) >> 2)] = (mtime % 1e3) * 1e3 * 1e3;
    HEAP64[(((buf) + (72)) >> 3)] = BigInt(Math.floor(ctime / 1e3));
    HEAPU32[(((buf) + (80)) >> 2)] = (ctime % 1e3) * 1e3 * 1e3;
    HEAP64[(((buf) + (88)) >> 3)] = BigInt(stat.ino);
    return 0;
  },
  writeStatFs(buf, stats) {
    HEAP32[(((buf) + (4)) >> 2)] = stats.bsize;
    HEAP32[(((buf) + (40)) >> 2)] = stats.bsize;
    HEAP32[(((buf) + (8)) >> 2)] = stats.blocks;
    HEAP32[(((buf) + (12)) >> 2)] = stats.bfree;
    HEAP32[(((buf) + (16)) >> 2)] = stats.bavail;
    HEAP32[(((buf) + (20)) >> 2)] = stats.files;
    HEAP32[(((buf) + (24)) >> 2)] = stats.ffree;
    HEAP32[(((buf) + (28)) >> 2)] = stats.fsid;
    HEAP32[(((buf) + (44)) >> 2)] = stats.flags;
    // ST_NOSUID
    HEAP32[(((buf) + (36)) >> 2)] = stats.namelen;
  },
  doMsync(addr, stream, len, flags, offset) {
    if (!FS.isFile(stream.node.mode)) {
      throw new FS.ErrnoError(43);
    }
    if (flags & 2) {
      // MAP_PRIVATE calls need not to be synced back to underlying fs
      return 0;
    }
    var buffer = HEAPU8.slice(addr, addr + len);
    FS.msync(stream, buffer, offset, len, flags);
  },
  getStreamFromFD(fd) {
    var stream = FS.getStreamChecked(fd);
    return stream;
  },
  varargs: undefined,
  getStr(ptr) {
    var ret = UTF8ToString(ptr);
    return ret;
  }
};

function _fd_close(fd) {
  try {
    var stream = SYSCALLS.getStreamFromFD(fd);
    FS.close(stream);
    return 0;
  } catch (e) {
    if (typeof FS == "undefined" || !(e.name === "ErrnoError")) throw e;
    return e.errno;
  }
}

/** @param {number=} offset */ var doReadv = (stream, iov, iovcnt, offset) => {
  var ret = 0;
  for (var i = 0; i < iovcnt; i++) {
    var ptr = HEAPU32[((iov) >> 2)];
    var len = HEAPU32[(((iov) + (4)) >> 2)];
    iov += 8;
    var curr = FS.read(stream, HEAP8, ptr, len, offset);
    if (curr < 0) return -1;
    ret += curr;
    if (curr < len) break;
    // nothing more to read
    if (typeof offset != "undefined") {
      offset += curr;
    }
  }
  return ret;
};

function _fd_read(fd, iov, iovcnt, pnum) {
  try {
    var stream = SYSCALLS.getStreamFromFD(fd);
    var num = doReadv(stream, iov, iovcnt);
    HEAPU32[((pnum) >> 2)] = num;
    return 0;
  } catch (e) {
    if (typeof FS == "undefined" || !(e.name === "ErrnoError")) throw e;
    return e.errno;
  }
}

function _fd_seek(fd, offset, whence, newOffset) {
  offset = bigintToI53Checked(offset);
  try {
    if (isNaN(offset)) return 61;
    var stream = SYSCALLS.getStreamFromFD(fd);
    FS.llseek(stream, offset, whence);
    HEAP64[((newOffset) >> 3)] = BigInt(stream.position);
    if (stream.getdents && offset === 0 && whence === 0) stream.getdents = null;
    // reset readdir state
    return 0;
  } catch (e) {
    if (typeof FS == "undefined" || !(e.name === "ErrnoError")) throw e;
    return e.errno;
  }
}

/** @param {number=} offset */ var doWritev = (stream, iov, iovcnt, offset) => {
  var ret = 0;
  for (var i = 0; i < iovcnt; i++) {
    var ptr = HEAPU32[((iov) >> 2)];
    var len = HEAPU32[(((iov) + (4)) >> 2)];
    iov += 8;
    var curr = FS.write(stream, HEAP8, ptr, len, offset);
    if (curr < 0) return -1;
    ret += curr;
    if (curr < len) {
      // No more space to write.
      break;
    }
    if (typeof offset != "undefined") {
      offset += curr;
    }
  }
  return ret;
};

function _fd_write(fd, iov, iovcnt, pnum) {
  try {
    var stream = SYSCALLS.getStreamFromFD(fd);
    var num = doWritev(stream, iov, iovcnt);
    HEAPU32[((pnum) >> 2)] = num;
    return 0;
  } catch (e) {
    if (typeof FS == "undefined" || !(e.name === "ErrnoError")) throw e;
    return e.errno;
  }
}

var _llvm_eh_typeid_for = type => type;

function _random_get(buffer, size) {
  try {
    randomFill(HEAPU8.subarray(buffer, buffer + size));
    return 0;
  } catch (e) {
    if (typeof FS == "undefined" || !(e.name === "ErrnoError")) throw e;
    return e.errno;
  }
}

/** @type {WebAssembly.Table} */ var wasmTable;

/** @suppress{checkTypes} */ var getWasmTableEntry = funcPtr => wasmTable.get(funcPtr);

FS.createPreloadedFile = FS_createPreloadedFile;

FS.staticInit();

// This error may happen quite a bit. To avoid overhead we reuse it (and
// suffer a lack of stack info).
MEMFS.doesNotExistError = new FS.ErrnoError(44);

/** @suppress {checkTypes} */ MEMFS.doesNotExistError.stack = "<generic error, no stack>";

// End JS library code
// include: postlibrary.js
// This file is included after the automatically-generated JS library code
// but before the wasm module is created.
{
  // Begin ATMODULES hooks
  if (Module["noExitRuntime"]) noExitRuntime = Module["noExitRuntime"];
  if (Module["preloadPlugins"]) preloadPlugins = Module["preloadPlugins"];
  if (Module["print"]) out = Module["print"];
  if (Module["printErr"]) err = Module["printErr"];
  if (Module["wasmBinary"]) wasmBinary = Module["wasmBinary"];
  // End ATMODULES hooks
  if (Module["arguments"]) arguments_ = Module["arguments"];
  if (Module["thisProgram"]) thisProgram = Module["thisProgram"];
}

// Begin runtime exports
// End runtime exports
// Begin JS library exports
// End JS library exports
// end include: postlibrary.js
// Imports from the Wasm binary.
var _openmpt_get_library_version, __ZN7openmpt19get_library_versionEv, _openmpt_get_core_version, __ZN7openmpt16get_core_versionEv, _openmpt_free_string, _free, _openmpt_get_string, __ZN7openmpt6string3getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, _openmpt_get_supported_extensions, _openmpt_is_extension_supported, _openmpt_log_func_default, _openmpt_log_func_silent, _openmpt_error_is_transient, _openmpt_error_string, _openmpt_error_func_default, _openmpt_error_func_log, _openmpt_error_func_store, _openmpt_error_func_ignore, _openmpt_error_func_errno, _openmpt_error_func_errno_userdata, _openmpt_could_open_probability, _openmpt_could_open_probability2, _openmpt_could_open_propability, _openmpt_probe_file_header_get_recommended_size, _openmpt_probe_file_header, _openmpt_probe_file_header_without_filesize, _openmpt_probe_file_header_from_stream, _openmpt_module_create, _openmpt_module_create2, _openmpt_module_create_from_memory, _openmpt_module_create_from_memory2, _openmpt_module_destroy, _openmpt_module_set_log_func, _openmpt_module_set_error_func, _openmpt_module_error_get_last, _openmpt_module_error_get_last_message, _openmpt_module_error_set_last, _openmpt_module_error_clear, _openmpt_module_select_subsong, _openmpt_module_get_selected_subsong, _openmpt_module_set_repeat_count, _openmpt_module_get_repeat_count, _openmpt_module_get_duration_seconds, _openmpt_module_set_position_seconds, _openmpt_module_get_position_seconds, _openmpt_module_set_position_order_row, _openmpt_module_get_render_param, _openmpt_module_set_render_param, _openmpt_module_read_mono, _openmpt_module_read_stereo, _openmpt_module_read_quad, _openmpt_module_read_float_mono, _openmpt_module_read_float_stereo, _openmpt_module_read_float_quad, _openmpt_module_read_interleaved_stereo, _openmpt_module_read_interleaved_quad, _openmpt_module_read_interleaved_float_stereo, _openmpt_module_read_interleaved_float_quad, _openmpt_module_get_metadata_keys, _openmpt_module_get_metadata, _openmpt_module_get_current_estimated_bpm, _openmpt_module_get_current_speed, _openmpt_module_get_current_tempo, _openmpt_module_get_current_tempo2, _openmpt_module_get_current_order, _openmpt_module_get_current_pattern, _openmpt_module_get_current_row, _openmpt_module_get_current_playing_channels, _openmpt_module_get_current_channel_vu_mono, _openmpt_module_get_current_channel_vu_left, _openmpt_module_get_current_channel_vu_right, _openmpt_module_get_current_channel_vu_rear_left, _openmpt_module_get_current_channel_vu_rear_right, _openmpt_module_get_num_subsongs, _openmpt_module_get_num_channels, _openmpt_module_get_num_orders, _openmpt_module_get_num_patterns, _openmpt_module_get_num_instruments, _openmpt_module_get_num_samples, _openmpt_module_get_subsong_name, _openmpt_module_get_channel_name, _openmpt_module_get_order_name, _openmpt_module_get_pattern_name, _openmpt_module_get_instrument_name, _openmpt_module_get_sample_name, _openmpt_module_get_order_pattern, _openmpt_module_get_pattern_num_rows, _openmpt_module_get_pattern_row_channel_command, _openmpt_module_format_pattern_row_channel_command, _openmpt_module_highlight_pattern_row_channel_command, _openmpt_module_format_pattern_row_channel, _openmpt_module_highlight_pattern_row_channel, _openmpt_module_get_ctls, _openmpt_module_ctl_get, _openmpt_module_ctl_get_boolean, _openmpt_module_ctl_get_integer, _openmpt_module_ctl_get_floatingpoint, _openmpt_module_ctl_get_text, _openmpt_module_ctl_set, _openmpt_module_ctl_set_boolean, _openmpt_module_ctl_set_integer, _openmpt_module_ctl_set_floatingpoint, _openmpt_module_ctl_set_text, _openmpt_module_ext_create, _openmpt_module_ext_create_from_memory, _openmpt_module_ext_destroy, _openmpt_module_ext_get_module, _openmpt_module_ext_get_interface, __ZN7openmpt9exceptionC2ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, __ZNK7openmpt9exception4whatEv, __ZN7openmpt9exceptionD2Ev, _malloc, __ZN7openmpt9exceptionC2ERKS0_, __ZN7openmpt9exceptionC2EOS0_, __ZN7openmpt9exceptionaSERKS0_, __ZN7openmpt9exceptionaSEOS0_, __ZN7openmpt9exceptionD0Ev, __ZN7openmpt24get_supported_extensionsEv, __ZN7openmpt22is_extension_supportedERKNSt3__212basic_stringIcNS0_11char_traitsIcEENS0_9allocatorIcEEEE, __ZN7openmpt23is_extension_supported2ENSt3__217basic_string_viewIcNS0_11char_traitsIcEEEE, __ZN7openmpt22could_open_probabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE, __ZN7openmpt22could_open_propabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE, __ZN7openmpt38probe_file_header_get_recommended_sizeEv, __ZN7openmpt17probe_file_headerEyPKSt4bytemy, __ZN7openmpt17probe_file_headerEyPKhmy, __ZN7openmpt17probe_file_headerEyPKSt4bytem, __ZN7openmpt17probe_file_headerEyPKhm, __ZN7openmpt17probe_file_headerEyRNSt3__213basic_istreamIcNS0_11char_traitsIcEEEE, __ZN7openmpt6moduleC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE, __ZN7openmpt6moduleC2EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE, __ZN7openmpt6moduleC2EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE, __ZN7openmpt6moduleC2EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleD2Ev, __ZN7openmpt6moduleD0Ev, __ZN7openmpt6module14select_subsongEi, __ZNK7openmpt6module20get_selected_subsongEv, __ZN7openmpt6module16set_repeat_countEi, __ZNK7openmpt6module16get_repeat_countEv, __ZNK7openmpt6module20get_duration_secondsEv, __ZN7openmpt6module20set_position_secondsEd, __ZNK7openmpt6module20get_position_secondsEv, __ZN7openmpt6module22set_position_order_rowEii, __ZNK7openmpt6module16get_render_paramEi, __ZN7openmpt6module16set_render_paramEii, __ZN7openmpt6module4readEimPs, __ZN7openmpt6module4readEimPsS1_, __ZN7openmpt6module4readEimPsS1_S1_S1_, __ZN7openmpt6module4readEimPf, __ZN7openmpt6module4readEimPfS1_, __ZN7openmpt6module4readEimPfS1_S1_S1_, __ZN7openmpt6module23read_interleaved_stereoEimPs, __ZN7openmpt6module21read_interleaved_quadEimPs, __ZN7openmpt6module23read_interleaved_stereoEimPf, __ZN7openmpt6module21read_interleaved_quadEimPf, __ZNK7openmpt6module17get_metadata_keysEv, __ZNK7openmpt6module12get_metadataERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, __ZNK7openmpt6module25get_current_estimated_bpmEv, __ZNK7openmpt6module17get_current_speedEv, __ZNK7openmpt6module17get_current_tempoEv, __ZNK7openmpt6module18get_current_tempo2Ev, __ZNK7openmpt6module17get_current_orderEv, __ZNK7openmpt6module19get_current_patternEv, __ZNK7openmpt6module15get_current_rowEv, __ZNK7openmpt6module28get_current_playing_channelsEv, __ZNK7openmpt6module27get_current_channel_vu_monoEi, __ZNK7openmpt6module27get_current_channel_vu_leftEi, __ZNK7openmpt6module28get_current_channel_vu_rightEi, __ZNK7openmpt6module32get_current_channel_vu_rear_leftEi, __ZNK7openmpt6module33get_current_channel_vu_rear_rightEi, __ZNK7openmpt6module16get_num_subsongsEv, __ZNK7openmpt6module16get_num_channelsEv, __ZNK7openmpt6module14get_num_ordersEv, __ZNK7openmpt6module16get_num_patternsEv, __ZNK7openmpt6module19get_num_instrumentsEv, __ZNK7openmpt6module15get_num_samplesEv, __ZNK7openmpt6module17get_subsong_namesEv, __ZNK7openmpt6module17get_channel_namesEv, __ZNK7openmpt6module15get_order_namesEv, __ZNK7openmpt6module17get_pattern_namesEv, __ZNK7openmpt6module20get_instrument_namesEv, __ZNK7openmpt6module16get_sample_namesEv, __ZNK7openmpt6module17get_order_patternEi, __ZNK7openmpt6module20get_pattern_num_rowsEi, __ZNK7openmpt6module31get_pattern_row_channel_commandEiiii, __ZNK7openmpt6module34format_pattern_row_channel_commandEiiii, __ZNK7openmpt6module37highlight_pattern_row_channel_commandEiiii, __ZNK7openmpt6module26format_pattern_row_channelEiiimb, __ZNK7openmpt6module29highlight_pattern_row_channelEiiimb, __ZNK7openmpt6module8get_ctlsEv, __ZNK7openmpt6module7ctl_getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, __ZNK7openmpt6module15ctl_get_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE, __ZNK7openmpt6module15ctl_get_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE, __ZNK7openmpt6module21ctl_get_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE, __ZNK7openmpt6module12ctl_get_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE, __ZN7openmpt6module7ctl_setERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEES9_, __ZN7openmpt6module15ctl_set_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEb, __ZN7openmpt6module15ctl_set_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEx, __ZN7openmpt6module21ctl_set_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEd, __ZN7openmpt6module12ctl_set_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEES5_, __ZN7openmpt10module_extC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE, __ZN7openmpt10module_extC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE, __ZN7openmpt10module_extC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE, __ZN7openmpt10module_extC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE, __ZN7openmpt10module_extC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt10module_extC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extD2Ev, __ZN7openmpt10module_extD0Ev, __ZN7openmpt10module_extC2ERKS0_, __ZN7openmpt10module_extaSERKS0_, __ZN7openmpt10module_ext13get_interfaceERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, __ZN7openmpt9exceptionC1ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE, __ZN7openmpt9exceptionC1ERKS0_, __ZN7openmpt9exceptionC1EOS0_, __ZN7openmpt9exceptionD1Ev, __ZN7openmpt6moduleC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE, __ZN7openmpt6moduleC1EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt6moduleC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE, __ZN7openmpt6moduleC1EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE, __ZN7openmpt6moduleC1EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt6moduleD1Ev, __ZN7openmpt10module_extC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE, __ZN7openmpt10module_extC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE, __ZN7openmpt10module_extC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE, __ZN7openmpt10module_extC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE, __ZN7openmpt10module_extC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE, __ZN7openmpt10module_extC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE, __ZN7openmpt10module_extD1Ev, __ZN7openmpt10module_extC1ERKS0_, __emscripten_timeout, _setThrew, __emscripten_tempret_set, __emscripten_stack_restore, _emscripten_stack_get_current, ___cxa_decrement_exception_refcount, ___cxa_increment_exception_refcount, ___cxa_can_catch, ___cxa_get_exception_ptr;

function assignWasmExports(wasmExports) {
  Module["_openmpt_get_library_version"] = _openmpt_get_library_version = wasmExports["za"];
  Module["__ZN7openmpt19get_library_versionEv"] = __ZN7openmpt19get_library_versionEv = wasmExports["Aa"];
  Module["_openmpt_get_core_version"] = _openmpt_get_core_version = wasmExports["Ba"];
  Module["__ZN7openmpt16get_core_versionEv"] = __ZN7openmpt16get_core_versionEv = wasmExports["Ca"];
  Module["_openmpt_free_string"] = _openmpt_free_string = wasmExports["Da"];
  Module["_free"] = _free = wasmExports["Ea"];
  Module["_openmpt_get_string"] = _openmpt_get_string = wasmExports["Fa"];
  Module["__ZN7openmpt6string3getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZN7openmpt6string3getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["Ga"];
  Module["_openmpt_get_supported_extensions"] = _openmpt_get_supported_extensions = wasmExports["Ha"];
  Module["_openmpt_is_extension_supported"] = _openmpt_is_extension_supported = wasmExports["Ia"];
  Module["_openmpt_log_func_default"] = _openmpt_log_func_default = wasmExports["Ja"];
  Module["_openmpt_log_func_silent"] = _openmpt_log_func_silent = wasmExports["Ka"];
  Module["_openmpt_error_is_transient"] = _openmpt_error_is_transient = wasmExports["La"];
  Module["_openmpt_error_string"] = _openmpt_error_string = wasmExports["Ma"];
  Module["_openmpt_error_func_default"] = _openmpt_error_func_default = wasmExports["Na"];
  Module["_openmpt_error_func_log"] = _openmpt_error_func_log = wasmExports["Oa"];
  Module["_openmpt_error_func_store"] = _openmpt_error_func_store = wasmExports["Pa"];
  Module["_openmpt_error_func_ignore"] = _openmpt_error_func_ignore = wasmExports["Qa"];
  Module["_openmpt_error_func_errno"] = _openmpt_error_func_errno = wasmExports["Ra"];
  Module["_openmpt_error_func_errno_userdata"] = _openmpt_error_func_errno_userdata = wasmExports["Sa"];
  Module["_openmpt_could_open_probability"] = _openmpt_could_open_probability = wasmExports["Ta"];
  Module["_openmpt_could_open_probability2"] = _openmpt_could_open_probability2 = wasmExports["Ua"];
  Module["_openmpt_could_open_propability"] = _openmpt_could_open_propability = wasmExports["Va"];
  Module["_openmpt_probe_file_header_get_recommended_size"] = _openmpt_probe_file_header_get_recommended_size = wasmExports["Wa"];
  Module["_openmpt_probe_file_header"] = _openmpt_probe_file_header = wasmExports["Xa"];
  Module["_openmpt_probe_file_header_without_filesize"] = _openmpt_probe_file_header_without_filesize = wasmExports["Ya"];
  Module["_openmpt_probe_file_header_from_stream"] = _openmpt_probe_file_header_from_stream = wasmExports["Za"];
  Module["_openmpt_module_create"] = _openmpt_module_create = wasmExports["_a"];
  Module["_openmpt_module_create2"] = _openmpt_module_create2 = wasmExports["$a"];
  Module["_openmpt_module_create_from_memory"] = _openmpt_module_create_from_memory = wasmExports["ab"];
  Module["_openmpt_module_create_from_memory2"] = _openmpt_module_create_from_memory2 = wasmExports["bb"];
  Module["_openmpt_module_destroy"] = _openmpt_module_destroy = wasmExports["cb"];
  Module["_openmpt_module_set_log_func"] = _openmpt_module_set_log_func = wasmExports["db"];
  Module["_openmpt_module_set_error_func"] = _openmpt_module_set_error_func = wasmExports["eb"];
  Module["_openmpt_module_error_get_last"] = _openmpt_module_error_get_last = wasmExports["fb"];
  Module["_openmpt_module_error_get_last_message"] = _openmpt_module_error_get_last_message = wasmExports["gb"];
  Module["_openmpt_module_error_set_last"] = _openmpt_module_error_set_last = wasmExports["hb"];
  Module["_openmpt_module_error_clear"] = _openmpt_module_error_clear = wasmExports["ib"];
  Module["_openmpt_module_select_subsong"] = _openmpt_module_select_subsong = wasmExports["jb"];
  Module["_openmpt_module_get_selected_subsong"] = _openmpt_module_get_selected_subsong = wasmExports["kb"];
  Module["_openmpt_module_set_repeat_count"] = _openmpt_module_set_repeat_count = wasmExports["lb"];
  Module["_openmpt_module_get_repeat_count"] = _openmpt_module_get_repeat_count = wasmExports["mb"];
  Module["_openmpt_module_get_duration_seconds"] = _openmpt_module_get_duration_seconds = wasmExports["nb"];
  Module["_openmpt_module_set_position_seconds"] = _openmpt_module_set_position_seconds = wasmExports["ob"];
  Module["_openmpt_module_get_position_seconds"] = _openmpt_module_get_position_seconds = wasmExports["pb"];
  Module["_openmpt_module_set_position_order_row"] = _openmpt_module_set_position_order_row = wasmExports["qb"];
  Module["_openmpt_module_get_render_param"] = _openmpt_module_get_render_param = wasmExports["rb"];
  Module["_openmpt_module_set_render_param"] = _openmpt_module_set_render_param = wasmExports["sb"];
  Module["_openmpt_module_read_mono"] = _openmpt_module_read_mono = wasmExports["tb"];
  Module["_openmpt_module_read_stereo"] = _openmpt_module_read_stereo = wasmExports["ub"];
  Module["_openmpt_module_read_quad"] = _openmpt_module_read_quad = wasmExports["vb"];
  Module["_openmpt_module_read_float_mono"] = _openmpt_module_read_float_mono = wasmExports["wb"];
  Module["_openmpt_module_read_float_stereo"] = _openmpt_module_read_float_stereo = wasmExports["xb"];
  Module["_openmpt_module_read_float_quad"] = _openmpt_module_read_float_quad = wasmExports["yb"];
  Module["_openmpt_module_read_interleaved_stereo"] = _openmpt_module_read_interleaved_stereo = wasmExports["zb"];
  Module["_openmpt_module_read_interleaved_quad"] = _openmpt_module_read_interleaved_quad = wasmExports["Ab"];
  Module["_openmpt_module_read_interleaved_float_stereo"] = _openmpt_module_read_interleaved_float_stereo = wasmExports["Bb"];
  Module["_openmpt_module_read_interleaved_float_quad"] = _openmpt_module_read_interleaved_float_quad = wasmExports["Cb"];
  Module["_openmpt_module_get_metadata_keys"] = _openmpt_module_get_metadata_keys = wasmExports["Db"];
  Module["_openmpt_module_get_metadata"] = _openmpt_module_get_metadata = wasmExports["Eb"];
  Module["_openmpt_module_get_current_estimated_bpm"] = _openmpt_module_get_current_estimated_bpm = wasmExports["Fb"];
  Module["_openmpt_module_get_current_speed"] = _openmpt_module_get_current_speed = wasmExports["Gb"];
  Module["_openmpt_module_get_current_tempo"] = _openmpt_module_get_current_tempo = wasmExports["Hb"];
  Module["_openmpt_module_get_current_tempo2"] = _openmpt_module_get_current_tempo2 = wasmExports["Ib"];
  Module["_openmpt_module_get_current_order"] = _openmpt_module_get_current_order = wasmExports["Jb"];
  Module["_openmpt_module_get_current_pattern"] = _openmpt_module_get_current_pattern = wasmExports["Kb"];
  Module["_openmpt_module_get_current_row"] = _openmpt_module_get_current_row = wasmExports["Lb"];
  Module["_openmpt_module_get_current_playing_channels"] = _openmpt_module_get_current_playing_channels = wasmExports["Mb"];
  Module["_openmpt_module_get_current_channel_vu_mono"] = _openmpt_module_get_current_channel_vu_mono = wasmExports["Nb"];
  Module["_openmpt_module_get_current_channel_vu_left"] = _openmpt_module_get_current_channel_vu_left = wasmExports["Ob"];
  Module["_openmpt_module_get_current_channel_vu_right"] = _openmpt_module_get_current_channel_vu_right = wasmExports["Pb"];
  Module["_openmpt_module_get_current_channel_vu_rear_left"] = _openmpt_module_get_current_channel_vu_rear_left = wasmExports["Qb"];
  Module["_openmpt_module_get_current_channel_vu_rear_right"] = _openmpt_module_get_current_channel_vu_rear_right = wasmExports["Rb"];
  Module["_openmpt_module_get_num_subsongs"] = _openmpt_module_get_num_subsongs = wasmExports["Sb"];
  Module["_openmpt_module_get_num_channels"] = _openmpt_module_get_num_channels = wasmExports["Tb"];
  Module["_openmpt_module_get_num_orders"] = _openmpt_module_get_num_orders = wasmExports["Ub"];
  Module["_openmpt_module_get_num_patterns"] = _openmpt_module_get_num_patterns = wasmExports["Vb"];
  Module["_openmpt_module_get_num_instruments"] = _openmpt_module_get_num_instruments = wasmExports["Wb"];
  Module["_openmpt_module_get_num_samples"] = _openmpt_module_get_num_samples = wasmExports["Xb"];
  Module["_openmpt_module_get_subsong_name"] = _openmpt_module_get_subsong_name = wasmExports["Yb"];
  Module["_openmpt_module_get_channel_name"] = _openmpt_module_get_channel_name = wasmExports["Zb"];
  Module["_openmpt_module_get_order_name"] = _openmpt_module_get_order_name = wasmExports["_b"];
  Module["_openmpt_module_get_pattern_name"] = _openmpt_module_get_pattern_name = wasmExports["$b"];
  Module["_openmpt_module_get_instrument_name"] = _openmpt_module_get_instrument_name = wasmExports["ac"];
  Module["_openmpt_module_get_sample_name"] = _openmpt_module_get_sample_name = wasmExports["bc"];
  Module["_openmpt_module_get_order_pattern"] = _openmpt_module_get_order_pattern = wasmExports["cc"];
  Module["_openmpt_module_get_pattern_num_rows"] = _openmpt_module_get_pattern_num_rows = wasmExports["dc"];
  Module["_openmpt_module_get_pattern_row_channel_command"] = _openmpt_module_get_pattern_row_channel_command = wasmExports["ec"];
  Module["_openmpt_module_format_pattern_row_channel_command"] = _openmpt_module_format_pattern_row_channel_command = wasmExports["fc"];
  Module["_openmpt_module_highlight_pattern_row_channel_command"] = _openmpt_module_highlight_pattern_row_channel_command = wasmExports["gc"];
  Module["_openmpt_module_format_pattern_row_channel"] = _openmpt_module_format_pattern_row_channel = wasmExports["hc"];
  Module["_openmpt_module_highlight_pattern_row_channel"] = _openmpt_module_highlight_pattern_row_channel = wasmExports["ic"];
  Module["_openmpt_module_get_ctls"] = _openmpt_module_get_ctls = wasmExports["jc"];
  Module["_openmpt_module_ctl_get"] = _openmpt_module_ctl_get = wasmExports["kc"];
  Module["_openmpt_module_ctl_get_boolean"] = _openmpt_module_ctl_get_boolean = wasmExports["lc"];
  Module["_openmpt_module_ctl_get_integer"] = _openmpt_module_ctl_get_integer = wasmExports["mc"];
  Module["_openmpt_module_ctl_get_floatingpoint"] = _openmpt_module_ctl_get_floatingpoint = wasmExports["nc"];
  Module["_openmpt_module_ctl_get_text"] = _openmpt_module_ctl_get_text = wasmExports["oc"];
  Module["_openmpt_module_ctl_set"] = _openmpt_module_ctl_set = wasmExports["pc"];
  Module["_openmpt_module_ctl_set_boolean"] = _openmpt_module_ctl_set_boolean = wasmExports["qc"];
  Module["_openmpt_module_ctl_set_integer"] = _openmpt_module_ctl_set_integer = wasmExports["rc"];
  Module["_openmpt_module_ctl_set_floatingpoint"] = _openmpt_module_ctl_set_floatingpoint = wasmExports["sc"];
  Module["_openmpt_module_ctl_set_text"] = _openmpt_module_ctl_set_text = wasmExports["tc"];
  Module["_openmpt_module_ext_create"] = _openmpt_module_ext_create = wasmExports["uc"];
  Module["_openmpt_module_ext_create_from_memory"] = _openmpt_module_ext_create_from_memory = wasmExports["vc"];
  Module["_openmpt_module_ext_destroy"] = _openmpt_module_ext_destroy = wasmExports["wc"];
  Module["_openmpt_module_ext_get_module"] = _openmpt_module_ext_get_module = wasmExports["xc"];
  Module["_openmpt_module_ext_get_interface"] = _openmpt_module_ext_get_interface = wasmExports["yc"];
  Module["__ZN7openmpt9exceptionC2ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZN7openmpt9exceptionC2ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["zc"];
  Module["__ZNK7openmpt9exception4whatEv"] = __ZNK7openmpt9exception4whatEv = wasmExports["Ac"];
  Module["__ZN7openmpt9exceptionD2Ev"] = __ZN7openmpt9exceptionD2Ev = wasmExports["Bc"];
  Module["_malloc"] = _malloc = wasmExports["Cc"];
  Module["__ZN7openmpt9exceptionC2ERKS0_"] = __ZN7openmpt9exceptionC2ERKS0_ = wasmExports["Dc"];
  Module["__ZN7openmpt9exceptionC2EOS0_"] = __ZN7openmpt9exceptionC2EOS0_ = wasmExports["Ec"];
  Module["__ZN7openmpt9exceptionaSERKS0_"] = __ZN7openmpt9exceptionaSERKS0_ = wasmExports["Fc"];
  Module["__ZN7openmpt9exceptionaSEOS0_"] = __ZN7openmpt9exceptionaSEOS0_ = wasmExports["Gc"];
  Module["__ZN7openmpt9exceptionD0Ev"] = __ZN7openmpt9exceptionD0Ev = wasmExports["Hc"];
  Module["__ZN7openmpt24get_supported_extensionsEv"] = __ZN7openmpt24get_supported_extensionsEv = wasmExports["Ic"];
  Module["__ZN7openmpt22is_extension_supportedERKNSt3__212basic_stringIcNS0_11char_traitsIcEENS0_9allocatorIcEEEE"] = __ZN7openmpt22is_extension_supportedERKNSt3__212basic_stringIcNS0_11char_traitsIcEENS0_9allocatorIcEEEE = wasmExports["Jc"];
  Module["__ZN7openmpt23is_extension_supported2ENSt3__217basic_string_viewIcNS0_11char_traitsIcEEEE"] = __ZN7openmpt23is_extension_supported2ENSt3__217basic_string_viewIcNS0_11char_traitsIcEEEE = wasmExports["Kc"];
  Module["__ZN7openmpt22could_open_probabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE"] = __ZN7openmpt22could_open_probabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE = wasmExports["Lc"];
  Module["__ZN7openmpt22could_open_propabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE"] = __ZN7openmpt22could_open_propabilityERNSt3__213basic_istreamIcNS0_11char_traitsIcEEEEdRNS0_13basic_ostreamIcS3_EE = wasmExports["Mc"];
  Module["__ZN7openmpt38probe_file_header_get_recommended_sizeEv"] = __ZN7openmpt38probe_file_header_get_recommended_sizeEv = wasmExports["Nc"];
  Module["__ZN7openmpt17probe_file_headerEyPKSt4bytemy"] = __ZN7openmpt17probe_file_headerEyPKSt4bytemy = wasmExports["Oc"];
  Module["__ZN7openmpt17probe_file_headerEyPKhmy"] = __ZN7openmpt17probe_file_headerEyPKhmy = wasmExports["Pc"];
  Module["__ZN7openmpt17probe_file_headerEyPKSt4bytem"] = __ZN7openmpt17probe_file_headerEyPKSt4bytem = wasmExports["Qc"];
  Module["__ZN7openmpt17probe_file_headerEyPKhm"] = __ZN7openmpt17probe_file_headerEyPKhm = wasmExports["Rc"];
  Module["__ZN7openmpt17probe_file_headerEyRNSt3__213basic_istreamIcNS0_11char_traitsIcEEEE"] = __ZN7openmpt17probe_file_headerEyRNSt3__213basic_istreamIcNS0_11char_traitsIcEEEE = wasmExports["Sc"];
  Module["__ZN7openmpt6moduleC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE = wasmExports["Tc"];
  Module["__ZN7openmpt6moduleC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE"] = __ZN7openmpt6moduleC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE = wasmExports["Uc"];
  Module["__ZN7openmpt6moduleC2EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC2EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["Vc"];
  Module["__ZN7openmpt6moduleC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["Wc"];
  Module["__ZN7openmpt6moduleC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE"] = __ZN7openmpt6moduleC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE = wasmExports["Xc"];
  Module["__ZN7openmpt6moduleC2EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC2EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Yc"];
  Module["__ZN7openmpt6moduleC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Zc"];
  Module["__ZN7openmpt6moduleC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE"] = __ZN7openmpt6moduleC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE = wasmExports["_c"];
  Module["__ZN7openmpt6moduleC2EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC2EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["$c"];
  Module["__ZN7openmpt6moduleC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["ad"];
  Module["__ZN7openmpt6moduleC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["bd"];
  Module["__ZN7openmpt6moduleD2Ev"] = __ZN7openmpt6moduleD2Ev = wasmExports["cd"];
  Module["__ZN7openmpt6moduleD0Ev"] = __ZN7openmpt6moduleD0Ev = wasmExports["dd"];
  Module["__ZN7openmpt6module14select_subsongEi"] = __ZN7openmpt6module14select_subsongEi = wasmExports["ed"];
  Module["__ZNK7openmpt6module20get_selected_subsongEv"] = __ZNK7openmpt6module20get_selected_subsongEv = wasmExports["fd"];
  Module["__ZN7openmpt6module16set_repeat_countEi"] = __ZN7openmpt6module16set_repeat_countEi = wasmExports["gd"];
  Module["__ZNK7openmpt6module16get_repeat_countEv"] = __ZNK7openmpt6module16get_repeat_countEv = wasmExports["hd"];
  Module["__ZNK7openmpt6module20get_duration_secondsEv"] = __ZNK7openmpt6module20get_duration_secondsEv = wasmExports["id"];
  Module["__ZN7openmpt6module20set_position_secondsEd"] = __ZN7openmpt6module20set_position_secondsEd = wasmExports["jd"];
  Module["__ZNK7openmpt6module20get_position_secondsEv"] = __ZNK7openmpt6module20get_position_secondsEv = wasmExports["kd"];
  Module["__ZN7openmpt6module22set_position_order_rowEii"] = __ZN7openmpt6module22set_position_order_rowEii = wasmExports["ld"];
  Module["__ZNK7openmpt6module16get_render_paramEi"] = __ZNK7openmpt6module16get_render_paramEi = wasmExports["md"];
  Module["__ZN7openmpt6module16set_render_paramEii"] = __ZN7openmpt6module16set_render_paramEii = wasmExports["nd"];
  Module["__ZN7openmpt6module4readEimPs"] = __ZN7openmpt6module4readEimPs = wasmExports["od"];
  Module["__ZN7openmpt6module4readEimPsS1_"] = __ZN7openmpt6module4readEimPsS1_ = wasmExports["pd"];
  Module["__ZN7openmpt6module4readEimPsS1_S1_S1_"] = __ZN7openmpt6module4readEimPsS1_S1_S1_ = wasmExports["qd"];
  Module["__ZN7openmpt6module4readEimPf"] = __ZN7openmpt6module4readEimPf = wasmExports["rd"];
  Module["__ZN7openmpt6module4readEimPfS1_"] = __ZN7openmpt6module4readEimPfS1_ = wasmExports["sd"];
  Module["__ZN7openmpt6module4readEimPfS1_S1_S1_"] = __ZN7openmpt6module4readEimPfS1_S1_S1_ = wasmExports["td"];
  Module["__ZN7openmpt6module23read_interleaved_stereoEimPs"] = __ZN7openmpt6module23read_interleaved_stereoEimPs = wasmExports["ud"];
  Module["__ZN7openmpt6module21read_interleaved_quadEimPs"] = __ZN7openmpt6module21read_interleaved_quadEimPs = wasmExports["vd"];
  Module["__ZN7openmpt6module23read_interleaved_stereoEimPf"] = __ZN7openmpt6module23read_interleaved_stereoEimPf = wasmExports["wd"];
  Module["__ZN7openmpt6module21read_interleaved_quadEimPf"] = __ZN7openmpt6module21read_interleaved_quadEimPf = wasmExports["xd"];
  Module["__ZNK7openmpt6module17get_metadata_keysEv"] = __ZNK7openmpt6module17get_metadata_keysEv = wasmExports["yd"];
  Module["__ZNK7openmpt6module12get_metadataERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZNK7openmpt6module12get_metadataERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["zd"];
  Module["__ZNK7openmpt6module25get_current_estimated_bpmEv"] = __ZNK7openmpt6module25get_current_estimated_bpmEv = wasmExports["Ad"];
  Module["__ZNK7openmpt6module17get_current_speedEv"] = __ZNK7openmpt6module17get_current_speedEv = wasmExports["Bd"];
  Module["__ZNK7openmpt6module17get_current_tempoEv"] = __ZNK7openmpt6module17get_current_tempoEv = wasmExports["Cd"];
  Module["__ZNK7openmpt6module18get_current_tempo2Ev"] = __ZNK7openmpt6module18get_current_tempo2Ev = wasmExports["Dd"];
  Module["__ZNK7openmpt6module17get_current_orderEv"] = __ZNK7openmpt6module17get_current_orderEv = wasmExports["Ed"];
  Module["__ZNK7openmpt6module19get_current_patternEv"] = __ZNK7openmpt6module19get_current_patternEv = wasmExports["Fd"];
  Module["__ZNK7openmpt6module15get_current_rowEv"] = __ZNK7openmpt6module15get_current_rowEv = wasmExports["Gd"];
  Module["__ZNK7openmpt6module28get_current_playing_channelsEv"] = __ZNK7openmpt6module28get_current_playing_channelsEv = wasmExports["Hd"];
  Module["__ZNK7openmpt6module27get_current_channel_vu_monoEi"] = __ZNK7openmpt6module27get_current_channel_vu_monoEi = wasmExports["Id"];
  Module["__ZNK7openmpt6module27get_current_channel_vu_leftEi"] = __ZNK7openmpt6module27get_current_channel_vu_leftEi = wasmExports["Jd"];
  Module["__ZNK7openmpt6module28get_current_channel_vu_rightEi"] = __ZNK7openmpt6module28get_current_channel_vu_rightEi = wasmExports["Kd"];
  Module["__ZNK7openmpt6module32get_current_channel_vu_rear_leftEi"] = __ZNK7openmpt6module32get_current_channel_vu_rear_leftEi = wasmExports["Ld"];
  Module["__ZNK7openmpt6module33get_current_channel_vu_rear_rightEi"] = __ZNK7openmpt6module33get_current_channel_vu_rear_rightEi = wasmExports["Md"];
  Module["__ZNK7openmpt6module16get_num_subsongsEv"] = __ZNK7openmpt6module16get_num_subsongsEv = wasmExports["Nd"];
  Module["__ZNK7openmpt6module16get_num_channelsEv"] = __ZNK7openmpt6module16get_num_channelsEv = wasmExports["Od"];
  Module["__ZNK7openmpt6module14get_num_ordersEv"] = __ZNK7openmpt6module14get_num_ordersEv = wasmExports["Pd"];
  Module["__ZNK7openmpt6module16get_num_patternsEv"] = __ZNK7openmpt6module16get_num_patternsEv = wasmExports["Qd"];
  Module["__ZNK7openmpt6module19get_num_instrumentsEv"] = __ZNK7openmpt6module19get_num_instrumentsEv = wasmExports["Rd"];
  Module["__ZNK7openmpt6module15get_num_samplesEv"] = __ZNK7openmpt6module15get_num_samplesEv = wasmExports["Sd"];
  Module["__ZNK7openmpt6module17get_subsong_namesEv"] = __ZNK7openmpt6module17get_subsong_namesEv = wasmExports["Td"];
  Module["__ZNK7openmpt6module17get_channel_namesEv"] = __ZNK7openmpt6module17get_channel_namesEv = wasmExports["Ud"];
  Module["__ZNK7openmpt6module15get_order_namesEv"] = __ZNK7openmpt6module15get_order_namesEv = wasmExports["Vd"];
  Module["__ZNK7openmpt6module17get_pattern_namesEv"] = __ZNK7openmpt6module17get_pattern_namesEv = wasmExports["Wd"];
  Module["__ZNK7openmpt6module20get_instrument_namesEv"] = __ZNK7openmpt6module20get_instrument_namesEv = wasmExports["Xd"];
  Module["__ZNK7openmpt6module16get_sample_namesEv"] = __ZNK7openmpt6module16get_sample_namesEv = wasmExports["Yd"];
  Module["__ZNK7openmpt6module17get_order_patternEi"] = __ZNK7openmpt6module17get_order_patternEi = wasmExports["Zd"];
  Module["__ZNK7openmpt6module20get_pattern_num_rowsEi"] = __ZNK7openmpt6module20get_pattern_num_rowsEi = wasmExports["_d"];
  Module["__ZNK7openmpt6module31get_pattern_row_channel_commandEiiii"] = __ZNK7openmpt6module31get_pattern_row_channel_commandEiiii = wasmExports["$d"];
  Module["__ZNK7openmpt6module34format_pattern_row_channel_commandEiiii"] = __ZNK7openmpt6module34format_pattern_row_channel_commandEiiii = wasmExports["ae"];
  Module["__ZNK7openmpt6module37highlight_pattern_row_channel_commandEiiii"] = __ZNK7openmpt6module37highlight_pattern_row_channel_commandEiiii = wasmExports["be"];
  Module["__ZNK7openmpt6module26format_pattern_row_channelEiiimb"] = __ZNK7openmpt6module26format_pattern_row_channelEiiimb = wasmExports["ce"];
  Module["__ZNK7openmpt6module29highlight_pattern_row_channelEiiimb"] = __ZNK7openmpt6module29highlight_pattern_row_channelEiiimb = wasmExports["de"];
  Module["__ZNK7openmpt6module8get_ctlsEv"] = __ZNK7openmpt6module8get_ctlsEv = wasmExports["ee"];
  Module["__ZNK7openmpt6module7ctl_getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZNK7openmpt6module7ctl_getERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["fe"];
  Module["__ZNK7openmpt6module15ctl_get_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE"] = __ZNK7openmpt6module15ctl_get_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE = wasmExports["ge"];
  Module["__ZNK7openmpt6module15ctl_get_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE"] = __ZNK7openmpt6module15ctl_get_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE = wasmExports["he"];
  Module["__ZNK7openmpt6module21ctl_get_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE"] = __ZNK7openmpt6module21ctl_get_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE = wasmExports["ie"];
  Module["__ZNK7openmpt6module12ctl_get_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE"] = __ZNK7openmpt6module12ctl_get_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEE = wasmExports["je"];
  Module["__ZN7openmpt6module7ctl_setERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEES9_"] = __ZN7openmpt6module7ctl_setERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEES9_ = wasmExports["ke"];
  Module["__ZN7openmpt6module15ctl_set_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEb"] = __ZN7openmpt6module15ctl_set_booleanENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEb = wasmExports["le"];
  Module["__ZN7openmpt6module15ctl_set_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEx"] = __ZN7openmpt6module15ctl_set_integerENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEx = wasmExports["me"];
  Module["__ZN7openmpt6module21ctl_set_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEd"] = __ZN7openmpt6module21ctl_set_floatingpointENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEEd = wasmExports["ne"];
  Module["__ZN7openmpt6module12ctl_set_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEES5_"] = __ZN7openmpt6module12ctl_set_textENSt3__217basic_string_viewIcNS1_11char_traitsIcEEEES5_ = wasmExports["oe"];
  Module["__ZN7openmpt10module_extC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt10module_extC2ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE = wasmExports["pe"];
  Module["__ZN7openmpt10module_extC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE"] = __ZN7openmpt10module_extC2ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE = wasmExports["qe"];
  Module["__ZN7openmpt10module_extC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE"] = __ZN7openmpt10module_extC2ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE = wasmExports["re"];
  Module["__ZN7openmpt10module_extC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE"] = __ZN7openmpt10module_extC2ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE = wasmExports["se"];
  Module["__ZN7openmpt10module_extC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC2EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["te"];
  Module["__ZN7openmpt10module_extC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC2EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["ue"];
  Module["__ZN7openmpt10module_extC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt10module_extC2EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["ve"];
  Module["__ZN7openmpt10module_extC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC2EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["we"];
  Module["__ZN7openmpt10module_extD2Ev"] = __ZN7openmpt10module_extD2Ev = wasmExports["xe"];
  Module["__ZN7openmpt10module_extD0Ev"] = __ZN7openmpt10module_extD0Ev = wasmExports["ye"];
  Module["__ZN7openmpt10module_extC2ERKS0_"] = __ZN7openmpt10module_extC2ERKS0_ = wasmExports["ze"];
  Module["__ZN7openmpt10module_extaSERKS0_"] = __ZN7openmpt10module_extaSERKS0_ = wasmExports["Ae"];
  Module["__ZN7openmpt10module_ext13get_interfaceERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZN7openmpt10module_ext13get_interfaceERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["Be"];
  Module["__ZN7openmpt9exceptionC1ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE"] = __ZN7openmpt9exceptionC1ERKNSt3__212basic_stringIcNS1_11char_traitsIcEENS1_9allocatorIcEEEE = wasmExports["Ce"];
  Module["__ZN7openmpt9exceptionC1ERKS0_"] = __ZN7openmpt9exceptionC1ERKS0_ = wasmExports["De"];
  Module["__ZN7openmpt9exceptionC1EOS0_"] = __ZN7openmpt9exceptionC1EOS0_ = wasmExports["Ee"];
  Module["__ZN7openmpt9exceptionD1Ev"] = __ZN7openmpt9exceptionD1Ev = wasmExports["Fe"];
  Module["__ZN7openmpt6moduleC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE = wasmExports["Ge"];
  Module["__ZN7openmpt6moduleC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE"] = __ZN7openmpt6moduleC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE = wasmExports["He"];
  Module["__ZN7openmpt6moduleC1EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC1EPKSt4byteS3_RNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["Ie"];
  Module["__ZN7openmpt6moduleC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt6moduleC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["Je"];
  Module["__ZN7openmpt6moduleC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE"] = __ZN7openmpt6moduleC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE = wasmExports["Ke"];
  Module["__ZN7openmpt6moduleC1EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC1EPKhS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Le"];
  Module["__ZN7openmpt6moduleC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Me"];
  Module["__ZN7openmpt6moduleC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE"] = __ZN7openmpt6moduleC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE = wasmExports["Ne"];
  Module["__ZN7openmpt6moduleC1EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC1EPKcS2_RNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Oe"];
  Module["__ZN7openmpt6moduleC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Pe"];
  Module["__ZN7openmpt6moduleC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt6moduleC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Qe"];
  Module["__ZN7openmpt6moduleD1Ev"] = __ZN7openmpt6moduleD1Ev = wasmExports["Re"];
  Module["__ZN7openmpt10module_extC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt10module_extC1ERNSt3__213basic_istreamIcNS1_11char_traitsIcEEEERNS1_13basic_ostreamIcS4_EERKNS1_3mapINS1_12basic_stringIcS4_NS1_9allocatorIcEEEESE_NS1_4lessISE_EENSC_INS1_4pairIKSE_SE_EEEEEE = wasmExports["Se"];
  Module["__ZN7openmpt10module_extC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE"] = __ZN7openmpt10module_extC1ERKNSt3__26vectorIhNS1_9allocatorIhEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_NS3_IcEEEESG_NS1_4lessISG_EENS3_INS1_4pairIKSG_SG_EEEEEE = wasmExports["Te"];
  Module["__ZN7openmpt10module_extC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE"] = __ZN7openmpt10module_extC1ERKNSt3__26vectorIcNS1_9allocatorIcEEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSA_S4_EESF_NS1_4lessISF_EENS3_INS1_4pairIKSF_SF_EEEEEE = wasmExports["Ue"];
  Module["__ZN7openmpt10module_extC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE"] = __ZN7openmpt10module_extC1ERKNSt3__26vectorISt4byteNS1_9allocatorIS3_EEEERNS1_13basic_ostreamIcNS1_11char_traitsIcEEEERKNS1_3mapINS1_12basic_stringIcSB_NS4_IcEEEESH_NS1_4lessISH_EENS4_INS1_4pairIKSH_SH_EEEEEE = wasmExports["Ve"];
  Module["__ZN7openmpt10module_extC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC1EPKhmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["We"];
  Module["__ZN7openmpt10module_extC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC1EPKcmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Xe"];
  Module["__ZN7openmpt10module_extC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE"] = __ZN7openmpt10module_extC1EPKSt4bytemRNSt3__213basic_ostreamIcNS4_11char_traitsIcEEEERKNS4_3mapINS4_12basic_stringIcS7_NS4_9allocatorIcEEEESE_NS4_4lessISE_EENSC_INS4_4pairIKSE_SE_EEEEEE = wasmExports["Ye"];
  Module["__ZN7openmpt10module_extC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE"] = __ZN7openmpt10module_extC1EPKvmRNSt3__213basic_ostreamIcNS3_11char_traitsIcEEEERKNS3_3mapINS3_12basic_stringIcS6_NS3_9allocatorIcEEEESD_NS3_4lessISD_EENSB_INS3_4pairIKSD_SD_EEEEEE = wasmExports["Ze"];
  Module["__ZN7openmpt10module_extD1Ev"] = __ZN7openmpt10module_extD1Ev = wasmExports["_e"];
  Module["__ZN7openmpt10module_extC1ERKS0_"] = __ZN7openmpt10module_extC1ERKS0_ = wasmExports["$e"];
  __emscripten_timeout = wasmExports["bf"];
  _setThrew = wasmExports["cf"];
  __emscripten_tempret_set = wasmExports["df"];
  __emscripten_stack_restore = wasmExports["ef"];
  _emscripten_stack_get_current = wasmExports["ff"];
  ___cxa_decrement_exception_refcount = wasmExports["gf"];
  ___cxa_increment_exception_refcount = wasmExports["hf"];
  ___cxa_can_catch = wasmExports["jf"];
  ___cxa_get_exception_ptr = wasmExports["kf"];
}

var wasmImports = {
  /** @export */ o: ___assert_fail,
  /** @export */ k: ___cxa_begin_catch,
  /** @export */ n: ___cxa_end_catch,
  /** @export */ ga: ___cxa_find_matching_catch_17,
  /** @export */ a: ___cxa_find_matching_catch_2,
  /** @export */ h: ___cxa_find_matching_catch_3,
  /** @export */ C: ___cxa_find_matching_catch_4,
  /** @export */ fa: ___cxa_find_matching_catch_6,
  /** @export */ Y: ___cxa_rethrow,
  /** @export */ x: ___cxa_throw,
  /** @export */ va: ___cxa_uncaught_exceptions,
  /** @export */ e: ___resumeException,
  /** @export */ qa: __abort_js,
  /** @export */ oa: __emscripten_runtime_keepalive_clear,
  /** @export */ pa: __setitimer_js,
  /** @export */ ma: __tzset_js,
  /** @export */ wa: _clock_time_get,
  /** @export */ Z: _emscripten_resize_heap,
  /** @export */ sa: _environ_get,
  /** @export */ ta: _environ_sizes_get,
  /** @export */ ia: _fd_close,
  /** @export */ ua: _fd_read,
  /** @export */ _: _fd_seek,
  /** @export */ ba: _fd_write,
  /** @export */ E: invoke_di,
  /** @export */ ha: invoke_did,
  /** @export */ Q: invoke_didi,
  /** @export */ H: invoke_dii,
  /** @export */ N: invoke_diii,
  /** @export */ P: invoke_diiii,
  /** @export */ U: invoke_fi,
  /** @export */ la: invoke_fii,
  /** @export */ y: invoke_i,
  /** @export */ d: invoke_ii,
  /** @export */ da: invoke_iid,
  /** @export */ ja: invoke_iifi,
  /** @export */ b: invoke_iii,
  /** @export */ V: invoke_iiifii,
  /** @export */ R: invoke_iiifiii,
  /** @export */ g: invoke_iiii,
  /** @export */ ea: invoke_iiiidd,
  /** @export */ p: invoke_iiiii,
  /** @export */ aa: invoke_iiiiid,
  /** @export */ t: invoke_iiiiii,
  /** @export */ w: invoke_iiiiiii,
  /** @export */ J: invoke_iiiiiiii,
  /** @export */ X: invoke_iiiiiiiii,
  /** @export */ G: invoke_iiiiiiiiiiii,
  /** @export */ $: invoke_iij,
  /** @export */ u: invoke_iiji,
  /** @export */ ca: invoke_ijiij,
  /** @export */ L: invoke_j,
  /** @export */ O: invoke_jiiii,
  /** @export */ j: invoke_v,
  /** @export */ l: invoke_vi,
  /** @export */ z: invoke_vid,
  /** @export */ c: invoke_vii,
  /** @export */ I: invoke_viid,
  /** @export */ S: invoke_viif,
  /** @export */ f: invoke_viii,
  /** @export */ ka: invoke_viiidi,
  /** @export */ i: invoke_viiii,
  /** @export */ q: invoke_viiiii,
  /** @export */ A: invoke_viiiiii,
  /** @export */ v: invoke_viiiiiii,
  /** @export */ T: invoke_viiiiiiii,
  /** @export */ W: invoke_viiiiiiiii,
  /** @export */ D: invoke_viiiiiiiiii,
  /** @export */ F: invoke_viiiiiiiiiiiiiii,
  /** @export */ m: invoke_viiij,
  /** @export */ s: invoke_viij,
  /** @export */ K: invoke_viiji,
  /** @export */ B: invoke_vij,
  /** @export */ M: invoke_viji,
  /** @export */ r: _llvm_eh_typeid_for,
  /** @export */ na: _proc_exit,
  /** @export */ ra: _random_get
};

var wasmExports;

createWasm();

function invoke_iii(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viii(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_vii(index, a1, a2) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_ii(index, a1) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiii(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiiiii(index, a1, a2, a3, a4, a5, a6) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_vi(index, a1) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_v(index) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)();
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiii(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiii(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiii(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_i(index) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)();
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_vij(index, a1, a2) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiii(index, a1, a2, a3, a4, a5, a6) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiji(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiij(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viij(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiiii(index, a1, a2, a3, a4, a5, a6, a7) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8, a9);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_di(index, a1) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiifii(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_fi(index, a1) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiiii(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viif(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_vid(index, a1, a2) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiifiii(index, a1, a2, a3, a4, a5, a6) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_fii(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_didi(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_diiii(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_jiiii(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
    return 0n;
  }
}

function invoke_viiidi(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiji(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iifi(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiiiiii(index, a1, a2, a3, a4, a5, a6, a7) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_diii(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_did(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viid(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_dii(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiidd(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iid(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_ijiij(index, a1, a2, a3, a4) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viji(index, a1, a2, a3) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_j(index) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)();
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
    return 0n;
  }
}

function invoke_iiiiid(index, a1, a2, a3, a4, a5) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iij(index, a1, a2) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_iiiiiiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11) {
  var sp = stackSave();
  try {
    return getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

function invoke_viiiiiiiiiiiiiii(index, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15) {
  var sp = stackSave();
  try {
    getWasmTableEntry(index)(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15);
  } catch (e) {
    stackRestore(sp);
    if (e !== e + 0) throw e;
    _setThrew(1, 0);
  }
}

// include: postamble.js
// === Auto-generated postamble setup entry stuff ===
function run() {
  if (runDependencies > 0) {
    dependenciesFulfilled = run;
    return;
  }
  preRun();
  // a preRun added a dependency, run will be called later
  if (runDependencies > 0) {
    dependenciesFulfilled = run;
    return;
  }
  function doRun() {
    // run may have just been called through dependencies being fulfilled just in this very frame,
    // or while the async setStatus time below was happening
    Module["calledRun"] = true;
    if (ABORT) return;
    initRuntime();
    Module["onRuntimeInitialized"]?.();
    postRun();
  }
  if (Module["setStatus"]) {
    Module["setStatus"]("Running...");
    setTimeout(() => {
      setTimeout(() => Module["setStatus"](""), 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}

function preInit() {
  if (Module["preInit"]) {
    if (typeof Module["preInit"] == "function") Module["preInit"] = [ Module["preInit"] ];
    while (Module["preInit"].length > 0) {
      Module["preInit"].shift()();
    }
  }
}

preInit();

run();
